@@ -46,8 +46,10 @@ class SiriGrammar(Grammar):
     # Keywords
     k_access = Keyword('access')
     k_active_handles = Keyword('active_handles')
+    k_active_tasks = Keyword('active_tasks')
     k_address = Keyword('address')
     k_after = Keyword('after')
+    k_all = Keyword('all')
     k_alter = Keyword('alter')
     k_and = Keyword('and')
     k_as = Keyword('as')
@@ -75,6 +77,7 @@ class SiriGrammar(Grammar):
     k_false = Keyword('false')
     k_fifo_files = Keyword('fifo_files')
     k_filter = Keyword('filter')
+    k_first = Keyword('first')
     k_float = Keyword('float')
     k_for = Keyword('for')
     k_from = Keyword('from')
@@ -83,6 +86,8 @@ class SiriGrammar(Grammar):
     k_group = Keyword('group')
     k_groups = Keyword('groups')
     k_help = Choice(Keyword('help'), Token('?'))
+    k_idle_percentage = Keyword('idle_percentage')
+    k_idle_time = Keyword('idle_time')
     k_info = Keyword('info')
     k_ignore_threshold = Keyword('ignore_threshold')
     k_insert = Keyword('insert')
@@ -92,6 +97,7 @@ class SiriGrammar(Grammar):
         Keyword('intersection'),
         most_greedy=False)
     k_ip_support = Keyword('ip_support')
+    k_last = Keyword('last')
     k_length = Keyword('length')
     k_libuv = Keyword('libuv')
     k_limit = Keyword('limit')
@@ -140,6 +146,7 @@ class SiriGrammar(Grammar):
     k_start = Keyword('start')
     k_startup_time = Keyword('startup_time')
     k_status = Keyword('status')
+    k_stddev = Keyword('stddev')
     k_string = Keyword('string')
     k_suffix = Keyword('suffix')
     k_sum = Keyword('sum')
@@ -254,7 +261,10 @@ class SiriGrammar(Grammar):
         k_status,
         # Remote properties
         k_active_handles,
+        k_active_tasks,
         k_fifo_files,
+        k_idle_percentage,
+        k_idle_time,
         k_log_level,
         k_max_open_files,
         k_mem_usage,
@@ -329,8 +339,11 @@ class SiriGrammar(Grammar):
     where_server = Sequence(k_where, Prio(
         Sequence(Choice(
             k_active_handles,
+            k_active_tasks,
             k_buffer_size,
             k_fifo_files,
+            k_idle_percentage,
+            k_idle_time,            
             k_port,
             k_pool,
             k_startup_time,
@@ -394,14 +407,21 @@ class SiriGrammar(Grammar):
         k_symmetric_difference,
         most_greedy=False)
 
+    series_all = Choice(Token('*'), k_all, most_greedy=False)
     series_name = Repeat(string, 1, 1)
     group_name = Repeat(r_grave_str, 1, 1)
     series_re = Repeat(r_regex, 1, 1)
     uuid = Choice(r_uuid_str, string, most_greedy=False)
     group_match = Repeat(r_grave_str, 1, 1)
     series_match = List(
-        Choice(series_name, group_match, series_re, most_greedy=False),
-        series_sep, 1)
+        Choice(
+            series_all,
+            series_name,
+            group_match,
+            series_re,
+            most_greedy=False),
+        series_sep,
+        1)
     limit_expr = Sequence(k_limit, int_expr)
 
     before_expr = Sequence(k_before, time_expr)
@@ -412,10 +432,9 @@ class SiriGrammar(Grammar):
     prefix_expr = Sequence(k_prefix, string)
     suffix_expr = Sequence(k_suffix, string)
 
-    f_points = Choice(
-        Token('*'),
-        k_points,
-        most_greedy=False)
+    f_all = Choice(Token('*'), k_all, most_greedy=False)
+
+    f_points = Repeat(k_points, 1, 1)  # DEPRECATED
 
     f_difference = Sequence(
         k_difference,
@@ -429,34 +448,44 @@ class SiriGrammar(Grammar):
         ')')
     f_mean = Sequence(
         k_mean,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_median = Sequence(
         k_median,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_median_low = Sequence(
         k_median_low,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_median_high = Sequence(
         k_median_high,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_sum = Sequence(
         k_sum,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_min = Sequence(
         k_min,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_max = Sequence(
         k_max,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_count = Sequence(
         k_count,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_variance = Sequence(
         k_variance,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
     f_pvariance = Sequence(
         k_pvariance,
-        '(', time_expr, ')')
+        '(', Optional(time_expr), ')')
+    f_stddev = Sequence(
+        k_stddev,
+        '(', Optional(time_expr), ')')
+    f_first = Sequence(
+        k_first,
+        '(', Optional(time_expr), ')')
+    f_last = Sequence(
+        k_last,
+        '(', Optional(time_expr), ')')
+
     f_filter = Sequence(
         k_filter,
         '(',
@@ -483,11 +512,14 @@ class SiriGrammar(Grammar):
             k_count,
             k_variance,
             k_pvariance,
+            k_stddev,
+            k_first,
+            k_last,
             most_greedy=False),
         ')')
 
     aggregate_functions = List(Choice(
-        f_points,
+        f_all,
         f_limit,
         f_mean,
         f_sum,
@@ -499,9 +531,13 @@ class SiriGrammar(Grammar):
         f_count,
         f_variance,
         f_pvariance,
+        f_stddev,
+        f_first,
+        f_last,
         f_difference,
         f_derivative,
         f_filter,
+        f_points,
         most_greedy=False), '=>', 1)
 
     select_aggregate = Sequence(
@@ -509,6 +545,8 @@ class SiriGrammar(Grammar):
         Optional(prefix_expr),
         Optional(suffix_expr))
 
+    select_aggregates = List(select_aggregate, ',', 1)
+
     merge_as = Sequence(
         k_merge,
         k_as,
@@ -678,7 +716,7 @@ class SiriGrammar(Grammar):
 
     select_stmt = Sequence(
         k_select,
-        List(select_aggregate, ',', 1),
+        select_aggregates,
         k_from,
         series_match,
         Optional(where_series),
@@ -691,6 +729,7 @@ class SiriGrammar(Grammar):
 
     show_stmt = Sequence(k_show, List(Choice(
         k_active_handles,
+        k_active_tasks,
         k_buffer_path,
         k_buffer_size,
         k_dbname,
@@ -699,6 +738,8 @@ class SiriGrammar(Grammar):
         k_duration_log,
         k_duration_num,
         k_fifo_files,
+        k_idle_percentage,
+        k_idle_time,            
         k_ip_support,
         k_libuv,
         k_list_limit,
