@@ -5,7 +5,7 @@
  * should be used with the libcleri module.
  *
  * Source class: SiriGrammar
- * Created at: 2017-08-14 15:06:00
+ * Created at: 2018-05-16 16:15:46
  */
 #ifndef CLERI_EXPORT_SIRI_GRAMMAR_GRAMMAR_H_
 #define CLERI_EXPORT_SIRI_GRAMMAR_GRAMMAR_H_
@@ -51,10 +51,13 @@ enum cleri_grammar_ids {
     CLERI_GID_DROP_SHARDS,
     CLERI_GID_DROP_STMT,
     CLERI_GID_DROP_USER,
+    CLERI_GID_F_ALL,
     CLERI_GID_F_COUNT,
     CLERI_GID_F_DERIVATIVE,
     CLERI_GID_F_DIFFERENCE,
     CLERI_GID_F_FILTER,
+    CLERI_GID_F_FIRST,
+    CLERI_GID_F_LAST,
     CLERI_GID_F_LIMIT,
     CLERI_GID_F_MAX,
     CLERI_GID_F_MEAN,
@@ -64,6 +67,7 @@ enum cleri_grammar_ids {
     CLERI_GID_F_MIN,
     CLERI_GID_F_POINTS,
     CLERI_GID_F_PVARIANCE,
+    CLERI_GID_F_STDDEV,
     CLERI_GID_F_SUM,
     CLERI_GID_F_VARIANCE,
     CLERI_GID_GRANT_STMT,
@@ -114,8 +118,10 @@ enum cleri_grammar_ids {
     CLERI_GID_INT_OPERATOR,
     CLERI_GID_K_ACCESS,
     CLERI_GID_K_ACTIVE_HANDLES,
+    CLERI_GID_K_ACTIVE_TASKS,
     CLERI_GID_K_ADDRESS,
     CLERI_GID_K_AFTER,
+    CLERI_GID_K_ALL,
     CLERI_GID_K_ALTER,
     CLERI_GID_K_AND,
     CLERI_GID_K_AS,
@@ -143,6 +149,7 @@ enum cleri_grammar_ids {
     CLERI_GID_K_FALSE,
     CLERI_GID_K_FIFO_FILES,
     CLERI_GID_K_FILTER,
+    CLERI_GID_K_FIRST,
     CLERI_GID_K_FLOAT,
     CLERI_GID_K_FOR,
     CLERI_GID_K_FROM,
@@ -151,12 +158,15 @@ enum cleri_grammar_ids {
     CLERI_GID_K_GROUP,
     CLERI_GID_K_GROUPS,
     CLERI_GID_K_HELP,
+    CLERI_GID_K_IDLE_PERCENTAGE,
+    CLERI_GID_K_IDLE_TIME,
     CLERI_GID_K_IGNORE_THRESHOLD,
     CLERI_GID_K_INFO,
     CLERI_GID_K_INSERT,
     CLERI_GID_K_INTEGER,
     CLERI_GID_K_INTERSECTION,
     CLERI_GID_K_IP_SUPPORT,
+    CLERI_GID_K_LAST,
     CLERI_GID_K_LENGTH,
     CLERI_GID_K_LIBUV,
     CLERI_GID_K_LIMIT,
@@ -205,6 +215,7 @@ enum cleri_grammar_ids {
     CLERI_GID_K_START,
     CLERI_GID_K_STARTUP_TIME,
     CLERI_GID_K_STATUS,
+    CLERI_GID_K_STDDEV,
     CLERI_GID_K_STRING,
     CLERI_GID_K_SUFFIX,
     CLERI_GID_K_SUM,
@@ -254,7 +265,9 @@ enum cleri_grammar_ids {
     CLERI_GID_R_UINTEGER,
     CLERI_GID_R_UUID_STR,
     CLERI_GID_SELECT_AGGREGATE,
+    CLERI_GID_SELECT_AGGREGATES,
     CLERI_GID_SELECT_STMT,
+    CLERI_GID_SERIES_ALL,
     CLERI_GID_SERIES_COLUMNS,
     CLERI_GID_SERIES_MATCH,
     CLERI_GID_SERIES_NAME,
