@@ -19,6 +19,7 @@
 #include <slist/slist.h>
 #include <stddef.h>
 #include <strextra/strextra.h>
+#include <math.h>
 
 #define AGGR_NEW                                    \
 if ((aggr = AGGREGATE_new(gid)) == NULL)            \
@@ -70,6 +71,10 @@ static siridb_points_t * AGGREGATE_filter(
         siridb_points_t * source,
         siridb_aggr_t * aggr,
         char * err_msg);
+static siridb_points_t * AGGREGATE_to_one(
+        siridb_points_t * source,
+        siridb_aggr_t * aggr,
+        char * err_msg);
 static siridb_points_t * AGGREGATE_group_by(
         siridb_points_t * source,
         siridb_aggr_t * aggr,
@@ -147,6 +152,24 @@ static int aggr_variance(
         siridb_aggr_t * aggr,
         char * err_msg);
 
+static int aggr_stddev(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr,
+        char * err_msg);
+
+static int aggr_first(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr,
+        char * err_msg);
+
+static int aggr_last(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr,
+        char * err_msg);
+
 /*
  * Initialize aggregates.
  */
@@ -170,6 +193,9 @@ void siridb_init_aggregates(void)
     AGGREGATES[CLERI_GID_F_PVARIANCE - F_OFFSET] = aggr_pvariance;
     AGGREGATES[CLERI_GID_F_SUM - F_OFFSET] = aggr_sum;
     AGGREGATES[CLERI_GID_F_VARIANCE - F_OFFSET] = aggr_variance;
+    AGGREGATES[CLERI_GID_F_STDDEV - F_OFFSET] = aggr_stddev;
+    AGGREGATES[CLERI_GID_F_FIRST - F_OFFSET] = aggr_first;
+    AGGREGATES[CLERI_GID_F_LAST - F_OFFSET] = aggr_last;
 }
 
 /*
@@ -256,6 +282,18 @@ slist_t * siridb_aggregate_list(cleri_children_t * children, char * err_msg)
                     aggr->gid = CLERI_GID_F_PVARIANCE;
                     break;
 
+                case CLERI_GID_K_STDDEV:
+                    aggr->gid = CLERI_GID_F_STDDEV;
+                    break;
+
+                case CLERI_GID_K_FIRST:
+                    aggr->gid = CLERI_GID_F_FIRST;
+                    break;
+
+                case CLERI_GID_K_LAST:
+                    aggr->gid = CLERI_GID_F_LAST;
+                    break;
+
                 default:
                     assert (0);
                     break;
@@ -348,6 +386,19 @@ slist_t * siridb_aggregate_list(cleri_children_t * children, char * err_msg)
             break;
 
         case CLERI_GID_F_DIFFERENCE:
+        case CLERI_GID_F_COUNT:
+        case CLERI_GID_F_MAX:
+        case CLERI_GID_F_MEAN:
+        case CLERI_GID_F_MEDIAN:
+        case CLERI_GID_F_MEDIAN_HIGH:
+        case CLERI_GID_F_MEDIAN_LOW:
+        case CLERI_GID_F_MIN:
+        case CLERI_GID_F_PVARIANCE:
+        case CLERI_GID_F_SUM:
+        case CLERI_GID_F_VARIANCE:
+        case CLERI_GID_F_STDDEV:
+        case CLERI_GID_F_FIRST:
+        case CLERI_GID_F_LAST:
             AGGR_NEW
             if (children->node->children->node->children->
                         next->next->next != NULL)
@@ -372,35 +423,12 @@ slist_t * siridb_aggregate_list(cleri_children_t * children, char * err_msg)
 
             break;
 
-        case CLERI_GID_F_COUNT:
-        case CLERI_GID_F_MAX:
-        case CLERI_GID_F_MEAN:
-        case CLERI_GID_F_MEDIAN:
-        case CLERI_GID_F_MEDIAN_HIGH:
-        case CLERI_GID_F_MEDIAN_LOW:
-        case CLERI_GID_F_MIN:
-        case CLERI_GID_F_PVARIANCE:
-        case CLERI_GID_F_SUM:
-        case CLERI_GID_F_VARIANCE:
-            AGGR_NEW
-            aggr->group_by = children->node->children->node->children->
-                    next->next->node->result;
-
-            if (!aggr->group_by)
-            {
-                sprintf(err_msg,
-                        "Group by time must be an integer value "
-                        "larger than zero.");
-                AGGREGATE_free(aggr);
-                siridb_aggregate_list_free(slist);
-                return NULL;
-            }
-
-            SLIST_APPEND
-
+        case CLERI_GID_F_ALL:
             break;
 
         case CLERI_GID_F_POINTS:
+            log_warning("Keyword 'points' is deprecated, "
+                        "use '*' or 'all' instead.");
             break;
 
         default:
@@ -431,6 +459,26 @@ void siridb_aggregate_list_free(slist_t * alist)
     free(alist);
 }
 
+/*
+ * Returns 1 (true) if at least one aggregation requires all points to be queried.
+ */
+int siridb_aggregate_can_skip(cleri_children_t * children)
+{
+    cleri_node_t * nd = \
+            children->node->children->node->children->node->children->node;
+
+    switch (nd->cl_obj->gid)
+    {
+    case CLERI_GID_F_COUNT:
+    case CLERI_GID_F_FIRST:
+    case CLERI_GID_F_LAST:
+        return nd->children->next->next->next == NULL;
+
+    default:
+        return 0;
+    }
+}
+
 /*
  * Return a new allocated points object or the same object as source.
  * In case of an error NULL is returned and an error message is set or a
@@ -441,7 +489,7 @@ siridb_points_t * siridb_aggregate_run(
         siridb_aggr_t * aggr,
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (source->len);
 #endif
 
@@ -467,8 +515,7 @@ siridb_points_t * siridb_aggregate_run(
         return AGGREGATE_filter(source, aggr, err_msg);
 
     default:
-        assert (0);
-        break;
+        return AGGREGATE_to_one(source, aggr, err_msg);
     }
 
     return NULL;
@@ -758,13 +805,16 @@ static siridb_points_t * AGGREGATE_filter(
                     i < source->len;
                     i++, spt++)
             {
-                if (cexpr_str_cmp(
-                        aggr->filter_opr,
-                        (const char *) spt->val.raw,
-                        (const char *) value.raw))
+                if (cexpr_str_cmp(aggr->filter_opr, spt->val.str, value.str))
                 {
                     dpt->ts = spt->ts;
-                    dpt->val = spt->val;
+                    dpt->val.str = strdup(spt->val.str);
+                    if (dpt->val.str == NULL)
+                    {
+                        sprintf(err_msg, "Memory allocation error.");
+                        siridb_points_free(points);
+                        return NULL;
+                    }
                     dpt++;
                 }
             }
@@ -831,6 +881,64 @@ static siridb_points_t * AGGREGATE_filter(
     return points;
 }
 
+static siridb_points_t * AGGREGATE_to_one(
+        siridb_points_t * source,
+        siridb_aggr_t * aggr,
+        char * err_msg)
+{
+    siridb_points_t * points;
+    /* get correct callback function */
+    AGGR_cb aggr_cb = AGGREGATES[aggr->gid - F_OFFSET];
+
+    /* create new points with max possible size after re-indexing */
+    switch(aggr->gid)
+    {
+    case CLERI_GID_F_MEAN:
+    case CLERI_GID_F_MEDIAN:
+    case CLERI_GID_F_PVARIANCE:
+    case CLERI_GID_F_VARIANCE:
+    case CLERI_GID_F_STDDEV:
+        points = siridb_points_new(1, TP_DOUBLE);
+        break;
+    case CLERI_GID_F_COUNT:
+        points = siridb_points_new(1, TP_INT);
+        break;
+    case CLERI_GID_F_MEDIAN_HIGH:
+    case CLERI_GID_F_MAX:
+    case CLERI_GID_F_MEDIAN_LOW:
+    case CLERI_GID_F_MIN:
+    case CLERI_GID_F_SUM:
+    case CLERI_GID_F_FIRST:
+    case CLERI_GID_F_LAST:
+        points = siridb_points_new(1, source->tp);
+        break;
+    default:
+        assert (0);
+        points = NULL;
+    }
+
+    if (points == NULL)
+    {
+        sprintf(err_msg, "Memory allocation error.");
+        return NULL;  /* signal is raised */
+    }
+
+    /* set time-stamp */
+    points->data->ts = source->data[
+        (aggr->gid == CLERI_GID_F_FIRST) ? 0 : (source->len - 1)].ts;
+
+    /* set value */
+    if (aggr_cb(points->data, source, aggr, err_msg))
+    {
+        /* error occurred, return NULL */
+        siridb_points_free(points);
+        return NULL;
+    }
+
+    points->len++;
+    return points;
+}
+
 static siridb_points_t * AGGREGATE_group_by(
         siridb_points_t * source,
         siridb_aggr_t * aggr,
@@ -863,6 +971,7 @@ static siridb_points_t * AGGREGATE_group_by(
     case CLERI_GID_F_MEDIAN:
     case CLERI_GID_F_PVARIANCE:
     case CLERI_GID_F_VARIANCE:
+    case CLERI_GID_F_STDDEV:
     case CLERI_GID_F_DERIVATIVE:
         points = siridb_points_new(max_sz, TP_DOUBLE);
         break;
@@ -875,6 +984,8 @@ static siridb_points_t * AGGREGATE_group_by(
     case CLERI_GID_F_MIN:
     case CLERI_GID_F_SUM:
     case CLERI_GID_F_DIFFERENCE:
+    case CLERI_GID_F_FIRST:
+    case CLERI_GID_F_LAST:
         points = siridb_points_new(max_sz, group.tp);
         break;
     default:
@@ -937,7 +1048,7 @@ static siridb_points_t * AGGREGATE_group_by(
             points->data = point;
         }
     }
-#ifdef DEBUG
+#if DEBUG
     else
     {
         /* if not smaller it must be equal */
@@ -964,7 +1075,7 @@ static int aggr_derivative(
         siridb_aggr_t * aggr,
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1010,7 +1121,7 @@ static int aggr_difference(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1068,7 +1179,7 @@ static int aggr_max(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1112,7 +1223,7 @@ static int aggr_mean(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1154,7 +1265,7 @@ static int aggr_median(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1198,7 +1309,7 @@ static int aggr_median_high(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1233,7 +1344,7 @@ static int aggr_median_low(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1268,7 +1379,7 @@ static int aggr_min(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1312,7 +1423,7 @@ static int aggr_pvariance(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1341,7 +1452,7 @@ static int aggr_sum(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1395,7 +1506,7 @@ static int aggr_variance(
         siridb_aggr_t * aggr __attribute__((unused)),
         char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (points->len);
 #endif
 
@@ -1418,3 +1529,105 @@ static int aggr_variance(
 
     return 0;
 }
+
+static int aggr_stddev(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr __attribute__((unused)),
+        char * err_msg)
+{
+#if DEBUG
+    assert (points->len);
+#endif
+
+    switch (points->tp)
+    {
+    case TP_STRING:
+        sprintf(err_msg, "Cannot use stddev() on string type.");
+        return -1;
+
+    case TP_INT:
+    case TP_DOUBLE:
+        point->val.real = (points->len > 1) ?
+                sqrt(siridb_variance(points) / (points->len - 1)) : 0.0;
+        break;
+
+    default:
+        assert (0);
+        break;
+    }
+
+    return 0;
+}
+
+static int aggr_first(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr __attribute__((unused)),
+        char * err_msg __attribute__((unused)))
+{
+#if DEBUG
+    assert (points->len);
+#endif
+    siridb_point_t * source = points->data;
+
+    switch (points->tp)
+    {
+    case TP_STRING:
+        point->ts = source->ts;
+        point->val.str = strdup(source->val.str);
+        if (point->val.str == NULL)
+        {
+            sprintf(err_msg, "Memory allocation error.");
+            return -1;
+        }
+        break;
+
+    case TP_INT:
+    case TP_DOUBLE:
+        point->val = source->val;
+        break;
+
+    default:
+        assert (0);
+        break;
+    }
+
+    return 0;
+}
+
+static int aggr_last(
+        siridb_point_t * point,
+        siridb_points_t * points,
+        siridb_aggr_t * aggr __attribute__((unused)),
+        char * err_msg __attribute__((unused)))
+{
+#if DEBUG
+    assert (points->len);
+#endif
+    siridb_point_t * source = points->data + (points->len - 1);
+
+    switch (points->tp)
+    {
+    case TP_STRING:
+        point->ts = source->ts;
+        point->val.str = strdup(source->val.str);
+        if (point->val.str == NULL)
+        {
+            sprintf(err_msg, "Memory allocation error.");
+            return -1;
+        }
+        break;
+
+    case TP_INT:
+    case TP_DOUBLE:
+        point->val = source->val;
+        break;
+
+    default:
+        assert (0);
+        break;
+    }
+
+    return 0;
+}
