@@ -17,11 +17,12 @@
 #include <siri/err.h>
 #include <unistd.h>
 #include <string.h>
+#include <strextra/strextra.h>
 
 #define MAX_ITERATE_MERGE_COUNT 1000
 #define POINTS_MAX_QSORT 250000
 #define RAW_VALUES_THRESHOLD 7
-#define ZIP_THRESHOLD 5
+#define DICT_SZ 0x3fff
 
 static unsigned char * POINTS_zip_raw(
         siridb_points_t * points,
@@ -41,6 +42,34 @@ static void POINTS_merge_and_sort(slist_t * plist, siridb_points_t * points);
 static void POINTS_simple_sort(siridb_points_t * points);
 static inline int POINTS_compare(const void * a, const void * b);
 static void POINTS_highest_and_merge(slist_t * plist, siridb_points_t * points);
+static size_t POINTS_strlen_check_ascii(const char * str, uint8_t * is_ascii);
+static void POINTS_output_literal(
+        size_t len,
+        uint8_t * pt,
+        uint8_t ** out,
+        uint8_t is_ascii);
+static void POINTS_output_match(
+        size_t offset,
+        size_t len,
+        uint8_t ** out,
+        uint8_t is_ascii);
+static void POINTS_zip_str(
+        uint8_t ** out,
+        uint8_t ** pt,
+        uint8_t * src,
+        uint8_t * s,
+        size_t n,
+        uint8_t is_ascii);
+static int POINTS_set_cinfo_size(uint16_t * cinfo, size_t * size);
+inline static uint16_t POINTS_hash(uint32_t h);
+static void POINTS_destroy(siridb_points_t * points);
+
+static uint8_t * dictionary[DICT_SZ + 1];
+
+void siridb_points_init(void)
+{
+    memset(dictionary, 0, sizeof(dictionary));
+}
 
 /*
  * Returns NULL and raises a SIGNAL in case an error has occurred.
@@ -57,7 +86,6 @@ siridb_points_t * siridb_points_new(size_t size, points_tp tp)
     {
         points->len = 0;
         points->tp = tp;
-        points->content = NULL;
         points->data =
                 (siridb_point_t *) malloc(sizeof(siridb_point_t) * size);
         if (points->data == NULL)
@@ -87,7 +115,6 @@ siridb_points_t * siridb_points_copy(siridb_points_t * points)
         size_t sz = sizeof(siridb_point_t) * points->len;
         cpoints->len = points->len;
         cpoints->tp = points->tp;
-        cpoints->content = NULL;
         cpoints->data = (siridb_point_t *) malloc(sz);
         if (cpoints->data == NULL)
         {
@@ -98,6 +125,14 @@ siridb_points_t * siridb_points_copy(siridb_points_t * points)
         {
             memcpy(cpoints->data, points->data, sz);
         }
+        if (points->tp == TP_STRING)
+        {
+            for (size_t i = 0; i < points->len; ++i)
+            {
+                (cpoints->data + i)->val.str =
+                        strdup((points->data + i)->val.str);
+            }
+        }
     }
     return cpoints;
 }
@@ -107,7 +142,13 @@ siridb_points_t * siridb_points_copy(siridb_points_t * points)
  */
 void siridb_points_free(siridb_points_t * points)
 {
-    free(points->content);
+    if (points->tp == TP_STRING)
+    {
+        for (size_t i = 0; i < points->len; ++i)
+        {
+            free((points->data + i)->val.str);
+        }
+    }
     free(points->data);
     free(points);
 }
@@ -168,8 +209,12 @@ int siridb_points_pack(siridb_points_t * points, qp_packer_t * packer)
             }
             break;
         case TP_STRING:
-            /* TODO: handle string type */
-            assert (0);
+            for (size_t i = 0; i < points->len; i++, point++)
+            {
+                qp_add_type(packer, QP_ARRAY2);
+                qp_add_int64(packer, (int64_t) point->ts);
+                qp_add_string(packer, point->val.str);
+            }
             break;
         }
     }
@@ -192,14 +237,32 @@ void siridb_points_ts_correction(siridb_points_t * points, double factor)
  */
 int siridb_points_raw_pack(siridb_points_t * points, qp_packer_t * packer)
 {
-    return -(qp_add_type(packer, QP_ARRAY_OPEN) ||
+    int rc;
+    size_t size;
+    unsigned char * data;
+    if (points->tp == TP_STRING)
+    {
+        uint16_t cinfo;
+        data = siridb_points_zip_string(points, 0, points->len, &cinfo, &size);
+    }
+    else
+    {
+        data = (unsigned char *) points->data;
+        size = points->len * sizeof(siridb_point_t);
+    }
+
+    rc = -(qp_add_type(packer, QP_ARRAY_OPEN) ||
             qp_add_int8(packer, points->tp) ||
             qp_add_int32(packer, points->len) ||
-            qp_add_raw(
-                packer,
-                (unsigned char *) points->data,
-                points->len * sizeof(siridb_point_t)) ||
+            qp_add_raw(packer, data, size) ||
             qp_add_type(packer, QP_ARRAY_CLOSE));
+
+    if (points->tp == TP_STRING)
+    {
+        free(data);
+    }
+
+    return rc;
 }
 
 /*
@@ -209,7 +272,7 @@ int siridb_points_raw_pack(siridb_points_t * points, qp_packer_t * packer)
  */
 siridb_points_t * siridb_points_merge(slist_t * plist, char * err_msg)
 {
-#ifdef DEBUG
+#if DEBUG
     assert (plist->len >= 2);
 #endif
     siridb_points_t * points;
@@ -217,7 +280,6 @@ siridb_points_t * siridb_points_merge(slist_t * plist, char * err_msg)
     size_t n = 0;
     size_t i;
     uint8_t int2double = 0;
-
     for (i = 0; i < plist->len; )
     {
         points = (siridb_points_t *) plist->data[i];
@@ -230,7 +292,7 @@ siridb_points_t * siridb_points_merge(slist_t * plist, char * err_msg)
                 return plist->data[i];
             }
             /* cleanup empty points */
-            siridb_points_free(plist->data[i]);
+            POINTS_destroy(plist->data[i]);
 
             /* shrink plist length and fill gap */
             plist->data[i] = plist->data[plist->len];
@@ -259,7 +321,7 @@ siridb_points_t * siridb_points_merge(slist_t * plist, char * err_msg)
         i++;
     }
 
-#ifdef DEBUG
+#if DEBUG
     /* we have at least one series left */
     assert (plist->len >= 1);
 #endif
@@ -278,7 +340,6 @@ siridb_points_t * siridb_points_merge(slist_t * plist, char * err_msg)
 
     points = siridb_points_new(n, (int2double) ? TP_DOUBLE : tpts->tp);
 
-
     if (points == NULL)
     {
         sprintf(err_msg, "Memory allocation error.");
@@ -341,7 +402,7 @@ unsigned char * siridb_points_zip_int(
         uint16_t * cinfo,
         size_t * size)
 {
-    if (end - start < ZIP_THRESHOLD)
+    if (end - start < POINTS_ZIP_THRESHOLD)
     {
         return POINTS_zip_raw(points, start, end, cinfo, size);
     }
@@ -389,18 +450,7 @@ unsigned char * siridb_points_zip_int(
         }
     }
 
-    switch (vcount)
-    {
-        case 1: *cinfo = 0x80; break;
-        case 2: *cinfo = 0xc0; break;
-        case 3: *cinfo = 0xe0; break;
-        case 4: *cinfo = 0xf0; break;
-        case 5: *cinfo = 0xf8; break;
-        case 6: *cinfo = 0xfc; break;
-        case 7: *cinfo = 0xfe; break;
-        case 8: *cinfo = 0xff; break;
-    }
-
+    *cinfo = 0xff00 >> vcount;
     shift = (tcount > shift) ? tcount : shift;
     *cinfo <<= 8;
     *cinfo |= tcount | (shift << 4);
@@ -463,6 +513,204 @@ unsigned char * siridb_points_zip_int(
     return bits;
 }
 
+unsigned char * siridb_points_zip_string(
+        siridb_points_t * points,
+        uint_fast32_t start,
+        uint_fast32_t end,
+        uint16_t * cinfo,
+        size_t * size)
+{
+    uint_fast32_t n = end - start;
+    if (n < POINTS_ZIP_THRESHOLD)
+    {
+        *size = sizeof(uint64_t);
+        return siridb_points_raw_string(points, start, end, cinfo, size);
+    }
+    size_t * sizes = (size_t *) malloc(sizeof(size_t) * n);
+    if (sizes == NULL)
+    {
+        return NULL;
+    }
+    uint8_t is_ascii = 0x80;
+    uint64_t tdiff = 0;
+    uint8_t * src, * out, * pt, * spt, * sout;
+    uint64_t mask;
+    size_t sz, m = n, i = end - 2;
+    uint32_t sz_src = 0;
+    siridb_point_t * point = points->data + i;
+    uint64_t ts = (point + 1)->ts - point->ts;
+    uint8_t tinfo = 0;
+    uint8_t shift = 0;
+    uint8_t ibit;
+
+    sz = POINTS_strlen_check_ascii((point + 1)->val.str, &is_ascii);
+    sizes[--m] = sz;
+    sz_src += sz;
+    sz = POINTS_strlen_check_ascii((point)->val.str, &is_ascii);
+    sizes[--m] = sz;
+    sz_src += sz;
+
+    while (i-- > start)
+    {
+        point--;
+        sz = POINTS_strlen_check_ascii(point->val.str, &is_ascii);
+        sizes[--m] = sz;
+        sz_src += sz;
+        tdiff |= ts ^ ((point + 1)->ts - point->ts);
+    }
+
+    for (i = 1, mask = 0xff; i <= 8; mask <<= 8, i++)
+    {
+        if (tdiff & mask)
+        {
+            tinfo = i;
+        }
+        else if (ts & mask)
+        {
+            shift = i;
+        }
+    }
+
+    shift = tinfo > shift ? tinfo : shift;
+    ibit = tinfo | (shift << 4);
+    /* calculate time-stamps size */
+    sz = 13 + shift + tinfo*(n - 2);
+
+    src = (uint8_t *) malloc(sz_src);
+    out = (uint8_t *) malloc(sz + sz_src + (is_ascii ? 0 : (n * 8)));
+    if (src == NULL || out == NULL)
+    {
+        goto failed;
+    }
+    pt = out;
+    sout = out + sz;
+    spt = src;
+
+    memcpy(pt, &ibit, sizeof(uint8_t));
+    pt++;
+    memcpy(pt, &sz_src, sizeof(uint32_t));
+    pt += sizeof(uint32_t);
+    memcpy(pt, &point->ts, sizeof(uint64_t));
+    pt += sizeof(uint64_t);
+
+    POINTS_zip_str(&sout, &spt, src, point->val.raw, sizes[m++], is_ascii);
+
+    for (; shift-- > tinfo; ++pt)
+    {
+        *pt = ts >> (shift * 8);
+    }
+    for (i = end; --i > start;)
+    {
+        point++;
+        ts = point->ts - (point - 1)->ts;
+
+        for (shift = tinfo; shift--; ++pt)
+        {
+            *pt = ts >> (shift * 8);
+        }
+
+        POINTS_zip_str(&sout, &spt, src, point->val.raw, sizes[m++], is_ascii);
+    }
+    *size = sout - out;
+    if (POINTS_set_cinfo_size(cinfo, size))
+    {
+        goto failed;
+    }
+
+    sout = (uint8_t *) realloc(out, *size);
+    if (sout == NULL)
+    {
+        goto failed;
+
+    }
+    out = sout;
+    free(sizes);
+    free(src);
+    return out;
+
+failed:
+    free(sizes);
+    free(src);
+    free(out);
+    return NULL;
+}
+
+unsigned char * siridb_points_raw_string(
+        siridb_points_t * points,
+        uint_fast32_t start,
+        uint_fast32_t end,
+        uint16_t * cinfo,
+        size_t * size)
+{
+    size_t ts_sz = *size;
+    *size = 0;
+
+    uint_fast32_t n = end - start;
+    size_t * sizes = (size_t *) malloc(sizeof(size_t) * n);
+    if (sizes == NULL)
+    {
+        return NULL;
+    }
+
+    size_t * psz = sizes;
+    unsigned char * pdata;
+
+    for (uint_fast32_t i = start; i < end; ++i, ++psz)
+    {
+        *psz = strlen(points->data[i].val.str) + 1;
+        *size += *psz;
+    }
+
+    if (POINTS_set_cinfo_size(cinfo, size))
+    {
+        free(sizes);
+        return NULL;
+    }
+
+    /* when uncompressed, time-stamp size is not included */
+    *size += n * ts_sz;
+
+    unsigned char * cdata = (unsigned char *) malloc(*size);
+    if (cdata == NULL)
+    {
+        free(sizes);
+        return NULL;
+    }
+
+    pdata = cdata;
+    psz = sizes;
+
+    switch (ts_sz)
+    {
+    case sizeof(uint32_t):
+        for (uint_fast32_t i = start; i < end; ++i)
+        {
+            uint32_t ts = points->data[i].ts;
+            memcpy(pdata, &ts, sizeof(uint32_t));
+            pdata += sizeof(uint32_t);
+        }
+        break;
+    case sizeof(uint64_t):
+        for (uint_fast32_t i = start; i < end; ++i)
+        {
+            memcpy(pdata, &points->data[i].ts, sizeof(uint64_t));
+            pdata += sizeof(uint64_t);
+        }
+        break;
+    default:
+        assert(0);
+    }
+
+    for (uint_fast32_t i = start; i < end; ++i, ++psz)
+    {
+        memcpy(pdata, points->data[i].val.str, *psz);
+        pdata += *psz;
+    }
+
+    free(sizes);
+
+    return cdata;
+}
 /*
  * Return NULL in case an error has occurred. This function destroys the
  * original points.
@@ -474,7 +722,7 @@ unsigned char * siridb_points_zip_double(
         uint16_t * cinfo,
         size_t * size)
 {
-    if (end - start < ZIP_THRESHOLD)
+    if (end - start < POINTS_ZIP_THRESHOLD)
     {
         return POINTS_zip_raw(points, start, end, cinfo, size);
     }
@@ -574,7 +822,7 @@ void siridb_points_unzip_int(
         uint64_t * end_ts,
         uint8_t has_overlap)
 {
-    if (len < ZIP_THRESHOLD)
+    if (len < POINTS_ZIP_THRESHOLD)
     {
         return POINTS_unzip_raw(
                 points, bits, len, start_ts, end_ts, has_overlap);
@@ -667,7 +915,7 @@ void siridb_points_unzip_double(
         uint64_t * end_ts,
         uint8_t has_overlap)
 {
-    if (len < ZIP_THRESHOLD)
+    if (len < POINTS_ZIP_THRESHOLD)
     {
         return POINTS_unzip_raw(
                 points, bits, len, start_ts, end_ts, has_overlap);
@@ -761,9 +1009,226 @@ void siridb_points_unzip_double(
     }
 }
 
+static size_t POINTS_dec_len(uint8_t **pt)
+{
+    size_t sz = 0;
+    uint8_t shift = 6;
+    uint8_t * b = *pt;
+
+    (*pt)++;
+    sz += *b & 0x3f;
+
+    if ((*b & 0x40) == 0)
+    {
+        return sz;
+    }
+
+    do
+    {
+        b = *pt;
+        (*pt)++;
+        sz |= (*b & 0x7f) << shift;
+        shift += 7;
+    } while ((*b & 0x80) != 0);
+
+    return sz;
+}
+
+static int POINTS_unpack_string(
+        siridb_point_t * point,
+        size_t n,
+        size_t offset,
+        uint8_t * src,
+        uint8_t * buf)
+{
+    assert (n);
+    uint8_t * pt = src;
+    uint8_t * sbuf = buf;
+    uint8_t is_ascii = (0x80 & (*pt)) ^ 0x80;
+    size_t i = 0;
+    while (1)
+    {
+        if (is_ascii ^ ((*pt) & 0x80))
+        {
+            /* literal */
+            size_t len = (is_ascii) ? 0 : POINTS_dec_len(&pt);
+
+            do
+            {
+                *buf = *pt;
+                ++pt;
+                if (!*buf)
+                {
+                    if (i >= offset)
+                    {
+                        point->val.str = strdup((char *)sbuf);
+                        if (point->val.str == NULL)
+                        {
+                            return -1;
+                        }
+                        ++point;
+                        if (!--n)
+                        {
+                            return 0;
+                        }
+                    }
+                    ++i;
+                    ++buf;
+                    sbuf = buf;
+                    break;
+                }
+                ++buf;
+            }
+            while ((is_ascii && ((~(*pt)) & 0x80)) || ((!is_ascii) && --len));
+        }
+        else
+        {
+            /* match */
+            size_t off = POINTS_dec_len(&pt);
+            size_t len = POINTS_dec_len(&pt);
+            while (len--)
+            {
+              *buf = *(buf - off);
+              if (!*buf)
+              {
+                  if (i >= offset)
+                  {
+                      point->val.str = strdup((char *)sbuf);
+                      if (point->val.str == NULL)
+                      {
+                          return -1;
+                      }
+                      ++point;
+                      if (!--n)
+                      {
+                          return 0;
+                      }
+                  }
+                  ++i;
+                  ++buf;
+                  sbuf = buf;
+              }
+              else
+              {
+                  ++buf;
+              }
+            }
+        }
+    }
+}
+
+int siridb_points_unzip_string_raw(
+        siridb_points_t * points,
+        uint8_t * bits,
+        uint16_t len)
+{
+    uint64_t * tpt;
+    tpt = (uint64_t *) bits;
+    char * cpt = (char *) (bits + (len * sizeof(uint64_t)));
+
+    for (; points->len < len; ++points->len, ++tpt)
+    {
+        size_t slen;
+        points->data[points->len].ts = *tpt;
+        points->data[points->len].val.str = strx_dup(cpt, &slen);
+        cpt += slen + 1;
+    }
+    return 0;
+}
+
+int siridb_points_unzip_string(
+        siridb_points_t * points,
+        uint8_t * bits,
+        uint16_t len,
+        uint64_t * start_ts,
+        uint64_t * end_ts,
+        uint8_t has_overlap)
+{
+    uint64_t ts, tmp, mask;
+    siridb_point_t * point = points->data + points->len;
+    uint32_t src_sz;
+    uint8_t * buf, * pt = bits;
+    uint8_t j, tcount, tshift = *pt;
+    size_t i, offset;
+
+    tcount =  *pt & 0xf;
+    tshift = *pt & 0xf0;
+    tshift >>= 4;
+    pt++;
+    offset = 13 + tshift + (tcount * (len - 2));
+
+    memcpy(&src_sz, pt, sizeof(uint32_t));
+    pt += sizeof(uint32_t);
+    memcpy(&point->ts, pt, sizeof(uint64_t));
+    pt += sizeof(uint64_t);
+
+    buf = (uint8_t *) malloc(src_sz);
+    if (buf == NULL)
+    {
+        return -1;
+    }
+
+    for (mask = 0; tshift-- > tcount; ++pt)
+    {
+        mask |= ((uint64_t) *pt) << (tshift * 8);
+    }
+
+    ts = point->ts;
+    uint16_t n = len;
+
+    for (i = n; (end_ts == NULL || ts < *end_ts) && --i; )
+    {
+        if (start_ts != NULL && ts < *start_ts)
+        {
+            --n;
+        }
+        else
+        {
+            ++point;
+        }
+
+        for (tmp = 0, j = tcount; j--; ++pt)
+        {
+            tmp <<= 8;
+            tmp |= *pt;
+        }
+        ts += tmp | mask;
+
+        point->ts = ts;
+    }
+
+    n -= i;
+
+    if (POINTS_unpack_string(
+            points->data + points->len, n, i, bits + offset, buf))
+    {
+        free(buf);
+        return -1;
+    }
+
+    if (has_overlap && points->len)
+    {
+        qp_via_t v;
+        point = points->data + points->len;
+        for (i = n; i--; ++point)
+        {
+            ts = point->ts;
+            v = point->val;
+            siridb_points_add_point(points, &ts, &v);
+        }
+    }
+    else
+    {
+        points->len += n;
+    }
+
+    free(buf);
+    return 0;
+}
+
 size_t siridb_points_get_size_zipped(uint16_t cinfo, uint16_t len)
 {
-    if (len < ZIP_THRESHOLD)
+    if (len < POINTS_ZIP_THRESHOLD)
     {
         return len * 16;
     }
@@ -893,7 +1358,7 @@ static void POINTS_highest_and_merge(slist_t * plist, siridb_points_t * points)
         }
         else
         {
-            siridb_points_free(*m);
+            POINTS_destroy(*m);
 
             if (--plist->len)
             {
@@ -906,7 +1371,7 @@ static void POINTS_highest_and_merge(slist_t * plist, siridb_points_t * points)
             usleep(3000);
         }
     }
-#ifdef DEBUG
+#if DEBUG
     /* size should be exactly zero */
     assert (n == 0);
 #endif
@@ -941,7 +1406,7 @@ static void POINTS_sort_while_merge(slist_t * plist, siridb_points_t * points)
 
         if (!(*m)->len--)
         {
-            siridb_points_free(*m);
+            POINTS_destroy(*m);
 
             if (--plist->len)
             {
@@ -954,7 +1419,7 @@ static void POINTS_sort_while_merge(slist_t * plist, siridb_points_t * points)
             usleep(3000);
         }
     }
-#ifdef DEBUG
+#if DEBUG
     /* size should be exactly zero */
     assert (n == 0);
 #endif
@@ -981,7 +1446,7 @@ static void POINTS_merge_and_sort(slist_t * plist, siridb_points_t * points)
 
             if (!(*m)->len--)
             {
-                siridb_points_free(*m);
+                POINTS_destroy(*m);
                 if (--plist->len)
                 {
                     *m = (siridb_points_t *) plist->data[plist->len];
@@ -995,7 +1460,7 @@ static void POINTS_merge_and_sort(slist_t * plist, siridb_points_t * points)
         }
     }
 
-#ifdef DEBUG
+#if DEBUG
     /* size should be exactly zero */
     assert (n == 0);
 #endif
@@ -1019,6 +1484,7 @@ static inline int POINTS_compare(const void * a, const void * b)
 {
     return (((siridb_point_t *) a)->ts - ((siridb_point_t *) b)->ts);
 }
+
 /*
  * Sort points by time-stamp.
  *
@@ -1053,3 +1519,164 @@ static void POINTS_simple_sort(siridb_points_t * points)
         }
     }
 }
+
+static size_t POINTS_strlen_check_ascii(const char * str, uint8_t * is_ascii)
+{
+    size_t sz;
+    if (*is_ascii)
+    {
+        const char * pt = str;
+        for (; *pt; ++pt)
+        {
+            *is_ascii &= (*pt) ^ 0x80;
+        }
+        sz = pt - str + 1;
+    }
+    else
+    {
+        sz = strlen(str) + 1;
+    }
+    return sz;
+}
+
+static void POINTS_output_literal(
+        size_t len,
+        uint8_t * pt,
+        uint8_t ** out,
+        uint8_t is_ascii)
+{
+    if (!is_ascii)
+    {
+        size_t n = len;
+        **out = 0x80 | ((n > 0x3f) << 6) | (n & 0x3f);
+        (*out)++;
+        n >>= 6;
+
+        while (n)
+        {
+            **out = ((n > 0x7f) << 7) | (n & 0x7f);
+            (*out)++;
+            n >>= 7;
+        }
+    }
+
+    while (len--)
+    {
+        **out = *pt;
+        (*out)++;
+        pt++;
+    }
+}
+
+static void POINTS_output_match(
+        size_t offset,
+        size_t len,
+        uint8_t ** out,
+        uint8_t is_ascii)
+{
+    **out = is_ascii | ((offset > 0x3f) << 6) | (offset & 0x3f);
+    (*out)++;
+    offset >>= 6;
+
+    while (offset)
+    {
+        **out = ((offset > 0x7f) << 7) | (offset & 0x7f);
+        (*out)++;
+        offset >>= 7;
+    }
+
+    **out = is_ascii | ((len > 0x3f) << 6) | (len & 0x3f);
+    (*out)++;
+    len >>= 6;
+
+    while (len)
+    {
+        **out = ((len > 0x7f) << 7) | (len & 0x7f);
+        (*out)++;
+        len >>= 7;
+    }
+}
+
+static void POINTS_zip_str(
+        uint8_t ** out,
+        uint8_t ** pt,
+        uint8_t * src,
+        uint8_t * s,
+        size_t n,
+        uint8_t is_ascii)
+{
+    uint8_t * match;
+    uint8_t * literal = *pt;
+    memcpy(*pt, s, n);
+    uint8_t * end = (*pt) + n;
+    uint8_t * tend = end - sizeof(uint32_t);
+    while (*pt < tend)
+    {
+        uint32_t * inp = (uint32_t *) *pt;
+        uint16_t idx = POINTS_hash(*inp);
+        match = dictionary[POINTS_hash(idx)];
+        dictionary[idx] = *pt;
+        if (match >= src && match < *pt && *((uint32_t *) match) == *inp)
+        {
+            if (literal < *pt)
+            {
+                POINTS_output_literal((*pt) - literal, literal, out, is_ascii);
+            }
+
+            uint8_t * p = (*pt) + 4;
+            match += 4;
+            while (p < end && *match == *p)
+            {
+                ++match;
+                ++p;
+            }
+
+            POINTS_output_match(p - match, p - (*pt), out, is_ascii);
+            literal = *pt = p;
+        }
+        else
+        {
+            ++(*pt);
+        }
+    }
+
+    if (literal < end)
+    {
+        POINTS_output_literal(end - literal, literal, out, is_ascii);
+    }
+    *pt = end;
+}
+
+static int POINTS_set_cinfo_size(uint16_t * cinfo, size_t * size)
+{
+    if (*size >= 0x8000)
+    {
+        if (*size > 0x2000000)
+        {
+            log_critical("Size too large: %zu", *size);
+            return -1;
+        }
+        *size = (!!((*size) & 0x3ff)) + ((*size) >> 10);
+        *cinfo = (*size) & 0x8000;
+        *size <<= 10;
+    }
+    else
+    {
+        *cinfo = *size;
+    }
+    return 0;
+}
+
+inline static uint16_t POINTS_hash(uint32_t h)
+{
+    return ((h >> 17) ^ (h & 0xffff)) & DICT_SZ;
+}
+
+/*
+ * Destroy points. (parsing NULL is NOT allowed)
+ */
+static void POINTS_destroy(siridb_points_t * points)
+{
+    free(points->data);
+    free(points);
+}
