@@ -21,6 +21,7 @@
 #include <siri/db/series.h>
 #include <siri/db/shard.h>
 #include <siri/db/shards.h>
+#include <siri/db/points.h>
 #include <siri/optimize.h>
 #include <siri/err.h>
 #include <siri/file/pointer.h>
@@ -29,6 +30,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <strextra/strextra.h>
+
 
 /* max read buffer size used for reading from index file */
 #define SIRIDB_SHARD_MAX_CHUNK_SZ 65536
@@ -37,7 +40,7 @@
 #define SHARD_GROW_SZ 131072
 
 /* shard schema (schemas below 20 are reserved for Python SiriDB) */
-#define SIRIDB_SHARD_SHEMA 20
+#define SIRIDB_SHARD_SHEMA 21
 
 /*
  * Header schema layout
@@ -67,7 +70,8 @@
  * 12   (uint16_t)  LEN
  * 14   (uint16_t)  (OPTIONAL COMPRESSION INFO)
  */
-#define IDX_NUM32_SZ 14  /* or 16 when compressed */
+#define IDX32_SZ 14  /* or 16 when log/compressed */
+#define IDX32E_SZ 16
 
 /* 0    (uint32_t)  SERIES_ID
  * 4    (uint64_t)  START_TS
@@ -75,23 +79,8 @@
  * 20   (uint16_t)  LEN
  * 22   (uint16_t)  (OPTIONAL COMPRESSION INFO)
  */
-#define IDX_NUM64_SZ 22  /* or 24 when compressed */
-
-/* 0    (uint32_t)  SERIES_ID
- * 4    (uint32_t)  START_TS
- * 8    (uint32_t)  END_TS
- * 12   (uint16_t)  LEN
- * 14   (uint16_t)  LOG_SZ
- */
-#define IDX_LOG32_SZ 16
-
-/* 0    (uint32_t)  SERIES_ID
- * 4    (uint64_t)  START_TS
- * 12   (uint64_t)  END_TS
- * 20   (uint16_t)  LEN
- * 22   (uint16_t)  LOG_SZ
- */
-#define IDX_LOG64_SZ 24
+#define IDX64_SZ 22  /* or 24 when log/compressed */
+#define IDX64E_SZ 24
 
 #define SHARD_STATUS_SIZE 8
 
@@ -106,6 +95,7 @@
  * Max 65535 since uint16_t is used to store this value
  */
 #define DEFAULT_MAX_CHUNK_SZ_NUM 800
+#define DEFAULT_MAX_CHUNK_SZ_LOG 128
 
 static const siridb_shard_flags_repr_t flags_map[SHARD_STATUS_SIZE] = {
         {.repr="indexed", .flag=SIRIDB_SHARD_HAS_INDEX},
@@ -123,24 +113,24 @@ const char shard_type_map[2][7] = {
         "log"
 };
 
-static ssize_t SHARD_apply_idx_num(
+static ssize_t SHARD_apply_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
         char * pt,
         size_t pos,
-        int is_num64);
-static int SHARD_get_idx_num(
+        int is_ts64);
+static int SHARD_get_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
-        int is_num64);
-static int SHARD_load_idx_num(
+        int is_ts64);
+static int SHARD_load_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
         FILE * fp,
-        int is_num64);
+        int is_ts64);
 static inline int SHARD_init_fn(siridb_t * siridb, siridb_shard_t * shard);
 static int SHARD_grow(siridb_shard_t * shard);
-static int SHARD_write_header(
+static size_t SHARD_write_header(
         siridb_t * siridb,
         siridb_series_t * series,
         siridb_points_t * points,
@@ -156,7 +146,7 @@ static int SHARD_remove(siridb_shard_t * shard);
  */
 int siridb_shard_load(siridb_t * siridb, uint64_t id)
 {
-    int is_num64;
+    int is_ts64;
     FILE * fp;
     off_t shard_sz;
     siridb_shard_t * shard = (siridb_shard_t *) malloc(sizeof(siridb_shard_t));
@@ -218,6 +208,17 @@ int siridb_shard_load(siridb_t * siridb, uint64_t id)
         return -1;
     }
 
+    uint8_t schema = (uint8_t) header[HEADER_SCHEMA];
+    if (schema > SIRIDB_SHARD_SHEMA)
+    {
+        fclose(fp);
+        log_critical(
+                "Shard file '%s' has schema '%u' which is not supported with "
+                "this version of SiriDB.", shard->fn, schema);
+        siridb_shard_decref(shard);
+        return -1;
+    }
+
     /* set shard type, flags and max_chunk_sz */
     shard->tp = (uint8_t) header[HEADER_TP];
     shard->flags = (uint8_t) header[HEADER_FLAGS] | SIRIDB_SHARD_IS_LOADING;
@@ -241,9 +242,10 @@ int siridb_shard_load(siridb_t * siridb, uint64_t id)
     switch (shard->tp)
     {
     case SIRIDB_SHARD_TP_NUMBER:
-        is_num64 = time_precision > SIRIDB_TIME_SECONDS;
+    case SIRIDB_SHARD_TP_LOG:
+        is_ts64 = time_precision > SIRIDB_TIME_SECONDS;
 
-        if (SHARD_get_idx_num(siridb, shard, is_num64))
+        if (SHARD_get_idx(siridb, shard, is_ts64))
         {
             fclose(fp);
             log_critical("Cannot read index for shard: '%s'", shard->fn);
@@ -261,15 +263,10 @@ int siridb_shard_load(siridb_t * siridb, uint64_t id)
                 return -1;
             }
 
-            SHARD_load_idx_num(siridb, shard, fp, is_num64);
+            SHARD_load_idx(siridb, shard, fp, is_ts64);
         }
         break;
 
-    case SIRIDB_SHARD_TP_LOG:
-        /* TODO: implement string type */
-        assert (0);
-        break;
-
     default:
         fclose(fp);
         log_critical("Unknown type shard file: '%s'", shard->fn);
@@ -325,7 +322,9 @@ siridb_shard_t *  siridb_shard_create(
     shard->replacing = replacing;
     shard->len = shard->size = HEADER_SIZE;
     shard->max_chunk_sz = (replacing == NULL) ?
-            DEFAULT_MAX_CHUNK_SZ_NUM : replacing->max_chunk_sz;
+            (tp == SIRIDB_SHARD_TP_NUMBER ?
+                    DEFAULT_MAX_CHUNK_SZ_NUM : DEFAULT_MAX_CHUNK_SZ_LOG) :
+                    replacing->max_chunk_sz;
 
     FILE * fp;
     if (SHARD_init_fn(siridb, shard) < 0)
@@ -470,9 +469,9 @@ int siridb_shard_status(char * str, siridb_shard_t * shard)
  * Writes an index and points to a shard. The return value is the position
  * where the points start in the shard file.
  *
- * If an error has occurred, EOF will be returned and a SIGNAL will be raised.
+ * If an error has occurred, 0 will be returned and a SIGNAL will be raised.
  */
-long int siridb_shard_write_points(
+size_t siridb_shard_write_points(
         siridb_t * siridb,
         siridb_series_t * series,
         siridb_shard_t * shard,
@@ -488,8 +487,7 @@ long int siridb_shard_write_points(
     unsigned char * cdata = NULL;
 
     uint_fast32_t i;
-    long int pos = EOF;
-    int header_sz;
+    size_t pos, header_sz;
 
     if (shard->fp->fp == NULL)
     {
@@ -497,7 +495,7 @@ long int siridb_shard_write_points(
         {
             ERR_FILE
             log_critical("Cannot open file '%s'", shard->fn);
-            return EOF;
+            return 0;
         }
     }
     fp = shard->fp->fp;
@@ -507,13 +505,21 @@ long int siridb_shard_write_points(
         cdata = siridb_points_zip(points, start, end, cinfo, &dsize);
         if (cdata == NULL)
         {
+            ERR_ALLOC
             log_critical("Memory allocation error while compressing points");
-            return -1;
+            return 0;
         }
     }
     else if (series->tp == TP_STRING)
     {
-        /* string.. */
+        dsize = siridb->time->ts_sz;
+        cdata = siridb_points_raw_string(points, start, end, cinfo, &dsize);
+        if (cdata == NULL)
+        {
+            ERR_ALLOC
+            log_critical("Memory allocation error while compressing points");
+            return 0;
+        }
     }
     else
     {
@@ -530,7 +536,7 @@ long int siridb_shard_write_points(
     if (fseeko(fp, shard->len, SEEK_SET))
     {
         log_critical("Seek error in: '%s'", shard->fn);
-        return -1;
+        return 0;
     }
 
     if (idx_fp == NULL || (shard->flags & SIRIDB_SHARD_HAS_NEW_VALUES))
@@ -559,50 +565,48 @@ long int siridb_shard_write_points(
         pos = shard->len;
     }
 
-    if (header_sz < 0)
+    if (!header_sz)
     {
         ERR_FILE
         log_critical(
                 "Cannot write index header for shard id %" PRIu64,
                 shard->id);
         free(cdata);
-        return EOF;
+        return 0;
     }
 
-    if (cdata != NULL)
+    if (cdata == NULL)
     {
-        long int rc = fwrite(cdata, dsize, 1, fp);
-        free(cdata);
-        if (rc != 1)
+        size_t p = 0;
+        size_t ts_sz = siridb->time->ts_sz;
+        cdata = (unsigned char *) malloc(dsize);
+        if (cdata == NULL)
         {
-            ERR_FILE
-            log_critical("Cannot write points to file '%s'", shard->fn);
-            return EOF;
+            ERR_ALLOC
+            log_critical("Memory allocation error while compressing points");
+            return 0;
         }
-    }
-    else
-    {
+
         for (i = start; i < end; i++)
         {
-            if (fwrite(&points->data[i].ts, siridb->time->ts_sz, 1, fp) != 1 ||
-                fwrite(&points->data[i].val, 8, 1, fp) != 1)
-            {
-                ERR_FILE
-                log_critical("Cannot write points to file '%s'", shard->fn);
-                return EOF;
-            }
+            memcpy(cdata + p, &points->data[i].ts, ts_sz);
+            p += ts_sz;
+            memcpy(cdata + p, &points->data[i].val, 8);
+            p += 8;
         }
     }
 
-    if (fflush(fp))
+    long int rc = fwrite(cdata, dsize, 1, fp);
+    free(cdata);
+
+    if (rc != 1 || fflush(fp))
     {
         ERR_FILE
-        log_critical("Cannot write flush file '%s'", shard->fn);
-        return EOF;
+        log_critical("Cannot write points to file '%s'", shard->fn);
+        return 0;
     }
 
     shard->len = pos + dsize;
-
     return pos;
 }
 
@@ -877,30 +881,308 @@ int siridb_shard_get_points_num_compressed(
     return 0;
 }
 
-/*
- * COPY from siridb_shard_get_points_num32
- */
-int siridb_shard_get_points_log32(
-        siridb_points_t * points __attribute__((unused)),
-        idx_t * idx __attribute__((unused)),
-        uint64_t * start_ts __attribute__((unused)),
-        uint64_t * end_ts __attribute__((unused)),
-        uint8_t has_overlap __attribute__((unused)))
+int siridb_shard_get_points_log_compressed(
+        siridb_points_t * points,
+        idx_t * idx,
+        uint64_t * start_ts,
+        uint64_t * end_ts,
+        uint8_t has_overlap)
 {
-    return -1;  /* dummy function */
+    int rc;
+
+    if (idx->len < POINTS_ZIP_THRESHOLD)
+    {
+        return siridb_shard_get_points_log64(
+                points, idx, start_ts, end_ts, has_overlap);
+    }
+
+    uint8_t * bits;
+    size_t size = siridb_points_get_size_log(idx->cinfo);
+
+    if (idx->shard->fp->fp == NULL)
+    {
+        if (siri_fopen(siri.fh, idx->shard->fp, idx->shard->fn, "r+"))
+        {
+            log_critical(
+                    "Cannot open file '%s', skip reading points",
+                    idx->shard->fn);
+            return -1;
+        }
+    }
+
+    bits = (uint8_t *) malloc(size);
+    if (bits == NULL)
+    {
+        free(bits);
+        log_critical("Memory allocation error");
+        return -1;
+    }
+
+    if (    fseeko(idx->shard->fp->fp, idx->pos, SEEK_SET) ||
+            fread(  bits,
+                    sizeof(uint8_t),
+                    size,
+                    idx->shard->fp->fp) != size)
+    {
+        if (idx->shard->flags & SIRIDB_SHARD_IS_CORRUPT)
+        {
+            log_error("Cannot read from shard id %" PRIu64, idx->shard->id);
+        }
+        else
+        {
+            log_critical(
+                    "Cannot read from shard id %" PRIu64
+                    ". The next optimize cycle "
+                    "will fix this shard but you might loose some data.",
+                    idx->shard->id);
+            idx->shard->flags |= SIRIDB_SHARD_IS_CORRUPT;
+        }
+        free(bits);
+        return -1;
+    }
+
+    rc = siridb_points_unzip_string(
+            points,
+            bits,
+            idx->len,
+            start_ts,
+            end_ts,
+            has_overlap && (idx->shard->flags & SIRIDB_SHARD_HAS_OVERLAP));
+
+    free(bits);
+
+    return rc;
 }
 
 /*
- * COPY from siridb_shard_get_points_num32
+ * COPY from siridb_shard_get_points_log64
+ */
+int siridb_shard_get_points_log32(
+        siridb_points_t * points,
+        idx_t * idx,
+        uint64_t * start_ts,
+        uint64_t * end_ts,
+        uint8_t has_overlap)
+{
+    uint32_t * tdata, * tpt;
+    char * cdata, * cpt;
+    size_t len = points->len + idx->len;
+    size_t dsize = siridb_points_get_size_log(idx->cinfo);
+
+    if (idx->shard->fp->fp == NULL)
+    {
+        if (siri_fopen(siri.fh, idx->shard->fp, idx->shard->fn, "r+"))
+        {
+            log_critical(
+                    "Cannot open file '%s', skip reading points",
+                    idx->shard->fn);
+            return -1;
+        }
+    }
+
+    tdata = (uint32_t *) malloc(sizeof(uint32_t) * idx->len);
+    cdata = (char *) malloc(dsize);
+    if (cdata == NULL || tdata == NULL)
+    {
+        free(tdata);
+        free(cdata);
+        log_critical("Memory allocation error");
+        return -1;
+    }
+
+    if (    fseeko(idx->shard->fp->fp, idx->pos, SEEK_SET) ||
+            fread(  tdata,
+                    sizeof(uint32_t),
+                    idx->len,
+                    idx->shard->fp->fp) != idx->len ||
+            fread(  cdata,
+                    sizeof(unsigned char),
+                    dsize,
+                    idx->shard->fp->fp) != dsize)
+    {
+        if (idx->shard->flags & SIRIDB_SHARD_IS_CORRUPT)
+        {
+            log_error("Cannot read from shard id %" PRIu64, idx->shard->id);
+        }
+        else
+        {
+            log_critical(
+                    "Cannot read from shard id %" PRIu64
+                    ". The next optimize cycle "
+                    "will fix this shard but you might loose some data.",
+                    idx->shard->id);
+            idx->shard->flags |= SIRIDB_SHARD_IS_CORRUPT;
+        }
+        free(tdata);
+        free(cdata);
+        return -1;
+    }
+
+    /* set pointer to start */
+    tpt = tdata;
+    cpt = cdata;
+
+    /* crop from start if needed */
+    if (start_ts != NULL)
+    {
+        for (; *tpt < *start_ts;)
+        {
+            tpt++;
+            for(; *cpt; ++cpt);
+            ++cpt;
+            len--;
+        }
+    }
+
+    /* crop from end if needed */
+    if (end_ts != NULL)
+    {
+        for (uint32_t * p = tdata + (idx->len - 1); *p >= *end_ts; --p, len--);
+    }
+
+    if (    has_overlap &&
+            points->len &&
+            (idx->shard->flags & SIRIDB_SHARD_HAS_OVERLAP))
+    {
+        for (; points->len < len; ++tpt)
+        {
+            size_t slen;
+            qp_via_t v;
+            v.str = strx_dup(cpt, &slen);
+            cpt += slen + 1;
+            uint64_t ts = *tpt;
+            siridb_points_add_point(points, &ts, &v);
+        }
+    }
+    else
+    {
+        for (; points->len < len; points->len++, ++tpt)
+        {
+            size_t slen;
+            points->data[points->len].ts = *tpt;
+            points->data[points->len].val.str = strx_dup(cpt, &slen);
+            cpt += slen + 1;
+        }
+    }
+
+    free(tdata);
+    free(cdata);
+    return 0;
+}
+
+/*
+ * COPY from siridb_shard_get_points_log32
  */
 int siridb_shard_get_points_log64(
-        siridb_points_t * points __attribute__((unused)),
-        idx_t * idx __attribute__((unused)),
-        uint64_t * start_ts __attribute__((unused)),
-        uint64_t * end_ts __attribute__((unused)),
-        uint8_t has_overlap __attribute__((unused)))
+        siridb_points_t * points,
+        idx_t * idx,
+        uint64_t * start_ts,
+        uint64_t * end_ts,
+        uint8_t has_overlap)
 {
-    return -1;  /* dummy function */
+    uint64_t * tdata, * tpt;
+    char * cdata, * cpt;
+    size_t len = points->len + idx->len;
+    size_t dsize = siridb_points_get_size_log(idx->cinfo);
+
+    if (idx->shard->fp->fp == NULL)
+    {
+        if (siri_fopen(siri.fh, idx->shard->fp, idx->shard->fn, "r+"))
+        {
+            log_critical(
+                    "Cannot open file '%s', skip reading points",
+                    idx->shard->fn);
+            return -1;
+        }
+    }
+
+    tdata = (uint64_t *) malloc(sizeof(uint64_t) * idx->len);
+    cdata = (char *) malloc(dsize);
+    if (cdata == NULL || tdata == NULL)
+    {
+        free(tdata);
+        free(cdata);
+        log_critical("Memory allocation error");
+        return -1;
+    }
+
+    if (    fseeko(idx->shard->fp->fp, idx->pos, SEEK_SET) ||
+            fread(  tdata,
+                    sizeof(uint64_t),
+                    idx->len,
+                    idx->shard->fp->fp) != idx->len ||
+            fread(  cdata,
+                    sizeof(unsigned char),
+                    dsize,
+                    idx->shard->fp->fp) != dsize)
+    {
+        if (idx->shard->flags & SIRIDB_SHARD_IS_CORRUPT)
+        {
+            log_error("Cannot read from shard id %" PRIu64, idx->shard->id);
+        }
+        else
+        {
+            log_critical(
+                    "Cannot read from shard id %" PRIu64
+                    ". The next optimize cycle "
+                    "will fix this shard but you might loose some data.",
+                    idx->shard->id);
+            idx->shard->flags |= SIRIDB_SHARD_IS_CORRUPT;
+        }
+        free(tdata);
+        free(cdata);
+        return -1;
+    }
+
+    /* set pointer to start */
+    tpt = tdata;
+    cpt = cdata;
+
+    /* crop from start if needed */
+    if (start_ts != NULL)
+    {
+        for (; *tpt < *start_ts;)
+        {
+            tpt++;
+            for(; *cpt; ++cpt);
+            ++cpt;
+            len--;
+        }
+    }
+
+    /* crop from end if needed */
+    if (end_ts != NULL)
+    {
+        for (uint64_t * p = tdata + (idx->len - 1); *p >= *end_ts; --p, len--);
+    }
+
+    if (    has_overlap &&
+            points->len &&
+            (idx->shard->flags & SIRIDB_SHARD_HAS_OVERLAP))
+    {
+        for (; points->len < len; ++tpt)
+        {
+            size_t slen;
+            qp_via_t v;
+            v.str = strx_dup(cpt, &slen);
+            cpt += slen + 1;
+            siridb_points_add_point(points, tpt, &v);
+        }
+    }
+    else
+    {
+        for (; points->len < len; points->len++, ++tpt)
+        {
+            size_t slen;
+            points->data[points->len].ts = *tpt;
+            points->data[points->len].val.str = strx_dup(cpt, &slen);
+            cpt += slen + 1;
+        }
+    }
+
+    free(tdata);
+    free(cdata);
+    return 0;
 }
 
 /*
@@ -999,7 +1281,7 @@ int siridb_shard_optimize(siridb_shard_t * shard, siridb_t * siridb)
 
         if (    !siri_err &&
                 siri.optimize->status != SIRI_OPTIMIZE_CANCELLED &&
-                shard->id % siridb->duration_num == series->mask &&
+                shard->id % duration == series->mask &&
                 (~series->flags & SIRIDB_SERIES_IS_DROPPED) &&
                 (~new_shard->flags & SIRIDB_SHARD_IS_REMOVED))
         {
@@ -1020,7 +1302,7 @@ int siridb_shard_optimize(siridb_shard_t * shard, siridb_t * siridb)
 
             /* make this sleep depending on the active_tasks
              * (50ms per active task) */
-            usleep( 50000 * siridb->active_tasks + 100 );
+            usleep( 50000 * siridb->tasks.active + 100 );
         }
 
         siridb_series_decref(series);
@@ -1248,7 +1530,7 @@ void siridb__shard_free(siridb_shard_t * shard)
     /* this will close the file, even when other references exist */
     siri_fp_decref(shard->fp);
 
-#ifdef DEBUG
+#if DEBUG
     log_debug("Free shard id: %" PRIu64, shard->id);
 #endif
 
@@ -1314,12 +1596,12 @@ static int SHARD_remove(siridb_shard_t * shard)
  * is simply ignored. In case the series id is not possible (invalid id),
  * then an log error is displayed and the return value will be -1.
  */
-static ssize_t SHARD_apply_idx_num(
+static ssize_t SHARD_apply_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
         char * pt,
         size_t pos,
-        int is_num64)
+        int is_ts64)
 {
     ssize_t size;
     uint16_t len;
@@ -1332,22 +1614,35 @@ static ssize_t SHARD_apply_idx_num(
     {
         return 0;
     }
-    len = *((uint16_t *) (pt + (is_num64 ? 20 : 12)));  // LEN POS IN INDEX
+
+    len = *((uint16_t *) (pt + (is_ts64 ? 20 : 12)));  // LEN POS IN INDEX
     series = imap_get(siridb->series_map, series_id);
 
-    if (shard->flags & SIRIDB_SHARD_IS_COMPRESSED)
+    if (shard->tp == SIRIDB_SHARD_TP_LOG)
     {
-        cinfo = *((uint16_t *)(pt + (is_num64 ? IDX_NUM64_SZ : IDX_NUM32_SZ)));
+        cinfo = *((uint16_t *)(pt + (is_ts64 ? IDX64_SZ : IDX32_SZ)));
+        size = (ssize_t) siridb_points_get_size_log(cinfo);
+        size += len * (
+                shard->flags & SIRIDB_SHARD_IS_COMPRESSED ?
+                        (len < POINTS_ZIP_THRESHOLD ?
+                                sizeof(uint64_t) :
+                                0) : is_ts64 ?
+                                        sizeof(uint64_t) :
+                                        sizeof(uint32_t));
+    }
+    else if (shard->flags & SIRIDB_SHARD_IS_COMPRESSED)
+    {
+        cinfo = *((uint16_t *)(pt + (is_ts64 ? IDX64_SZ : IDX32_SZ)));
         size = (ssize_t) siridb_points_get_size_zipped(cinfo, len);
     }
     else
     {
-        size = len * (is_num64 ? 16 : 12);
+        size = len * (is_ts64 ? 16 : 12);
     }
 
     if (series == NULL)
     {
-        if (!series_id || series_id > siridb->max_series_id)
+        if (series_id > siridb->max_series_id)
         {
             log_error(
                     "Unexpected Series ID %" PRIu32
@@ -1382,10 +1677,10 @@ static ssize_t SHARD_apply_idx_num(
         if (siridb_series_add_idx(
                 series,
                 shard,
-                is_num64 ? // START_TS IN HEADER
+                is_ts64 ? // START_TS IN HEADER
                         (uint64_t) *((uint64_t *) (pt + 4)) :
                         (uint64_t) *((uint32_t *) (pt + 4)),
-                is_num64 ? // END_TS IN HEADER
+                is_ts64 ? // END_TS IN HEADER
                         (uint64_t) *((uint64_t *) (pt + 12)) :
                         (uint64_t) *((uint32_t *) (pt + 8)),
                 (uint32_t) pos,
@@ -1412,14 +1707,16 @@ static ssize_t SHARD_apply_idx_num(
  *
  * Member shard->len will be updated according the index.
  */
-static int SHARD_get_idx_num(
+static int SHARD_get_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
-        int is_num64)
+        int is_ts64)
 {
-    const unsigned int idx_sz = (shard->flags & SIRIDB_SHARD_IS_COMPRESSED) ?
-            (is_num64 ? 24 : 16) :
-            (is_num64 ? IDX_NUM64_SZ : IDX_NUM32_SZ);
+    const unsigned int idx_sz = (
+            (shard->flags & SIRIDB_SHARD_IS_COMPRESSED) ||
+            (shard->tp == SIRIDB_SHARD_TP_LOG)) ?
+                (is_ts64 ? IDX64E_SZ : IDX32E_SZ) :
+                (is_ts64 ? IDX64_SZ : IDX32_SZ);
     size_t i, n;
     char * data, * pt;
     ssize_t size;
@@ -1456,12 +1753,12 @@ static int SHARD_get_idx_num(
         pt = data;
         for (i = 0; i < n; i++, pt += idx_sz)
         {
-            size = SHARD_apply_idx_num(
+            size = SHARD_apply_idx(
                     siridb,
                     shard,
                     pt,
                     shard->len,
-                    is_num64);
+                    is_ts64);
 
             if (size < 0)
             {
@@ -1488,15 +1785,17 @@ static int SHARD_get_idx_num(
  * corrupt in case of disk errors and try to recover on the next optimize
  * cycle.
  */
-static int SHARD_load_idx_num(
+static int SHARD_load_idx(
         siridb_t * siridb,
         siridb_shard_t * shard,
         FILE * fp,
-        int is_num64)
+        int is_ts64)
 {
-    const unsigned int idx_sz = (shard->flags & SIRIDB_SHARD_IS_COMPRESSED) ?
-            is_num64 ? 24 : 16 :
-            is_num64 ? IDX_NUM64_SZ : IDX_NUM32_SZ;
+    const unsigned int idx_sz = (
+            (shard->flags & SIRIDB_SHARD_IS_COMPRESSED) ||
+            (shard->tp == SIRIDB_SHARD_TP_LOG)) ?
+                is_ts64 ? IDX64E_SZ : IDX32E_SZ :
+                is_ts64 ? IDX64_SZ : IDX32_SZ;
 
     char idx[idx_sz];
     ssize_t sz;
@@ -1507,7 +1806,7 @@ static int SHARD_load_idx_num(
     {
         pos = shard->len + idx_sz;
 
-        sz = SHARD_apply_idx_num(siridb, shard, idx, pos, is_num64);
+        sz = SHARD_apply_idx(siridb, shard, idx, pos, is_ts64);
         if (sz == 0)
         {
             break;
@@ -1559,10 +1858,10 @@ static inline int SHARD_init_fn(siridb_t * siridb, siridb_shard_t * shard)
  * Write a header for a chunk of points. The header can be written to argument
  * fp which should be a pointer to the index, or the shard file.
  *
- * In case of an error the function return EOF, otherwise the size which is
+ * In case of an error the function returns 0, otherwise the size which is
  * written.
  */
-static int SHARD_write_header(
+static size_t SHARD_write_header(
         siridb_t * siridb,
         siridb_series_t * series,
         siridb_points_t * points,
@@ -1572,12 +1871,9 @@ static int SHARD_write_header(
         FILE * fp)
 {
     uint16_t len = end - start;
-    int size = EOF;
-
-    if (fwrite(&series->id, sizeof(uint32_t), 1, fp) != 1)
-    {
-        return EOF;
-    }
+    size_t size = sizeof(uint32_t);
+    char buf[24];
+    memcpy(buf, &series->id, sizeof(uint32_t));
 
     switch (siridb->time->ts_sz)
     {
@@ -1585,24 +1881,18 @@ static int SHARD_write_header(
         {
             uint32_t start_ts = (uint32_t) points->data[start].ts;
             uint32_t end_ts = (uint32_t) points->data[end - 1].ts;
-            if (fwrite(&start_ts, sizeof(uint32_t), 1, fp) != 1 ||
-                fwrite(&end_ts, sizeof(uint32_t), 1, fp) != 1)
-            {
-                return EOF;
-            }
+            memcpy(buf + size, &start_ts, sizeof(uint32_t));
+            size += sizeof(uint32_t);
+            memcpy(buf + size, &end_ts, sizeof(uint32_t));
+            size += sizeof(uint32_t);
         }
-        /* TODO: this is not LOG compatible */
-        size = IDX_NUM32_SZ;
         break;
 
     case sizeof(uint64_t):
-        if (fwrite(&points->data[start].ts, sizeof(uint64_t), 1, fp) != 1 ||
-            fwrite(&points->data[end - 1].ts, sizeof(uint64_t), 1, fp) != 1)
-        {
-            return EOF;
-        }
-        /* TODO: this is not LOG compatible */
-        size = IDX_NUM64_SZ;
+        memcpy(buf + size, &points->data[start].ts, sizeof(uint64_t));
+        size += sizeof(uint64_t);
+        memcpy(buf + size, &points->data[end - 1].ts, sizeof(uint64_t));
+        size += sizeof(uint64_t);
         break;
 
     default:
@@ -1610,19 +1900,18 @@ static int SHARD_write_header(
         break;
     }
 
-
-    if (fwrite(&len, sizeof(uint16_t), 1, fp) != 1)
-    {
-        return EOF;
-    }
+    memcpy(buf + size, &len, sizeof(uint16_t));
+    size += sizeof(uint16_t);
 
     if (cinfo != NULL)
     {
+        memcpy(buf + size, cinfo, sizeof(uint16_t));
         size += sizeof(uint16_t);
-        if (fwrite(cinfo, sizeof(uint16_t), 1, fp) != 1)
-        {
-            return EOF;
-        }
+    }
+
+    if (fwrite(buf, size, 1, fp) != 1)
+    {
+        return 0;
     }
 
     return size;
