@@ -5,7 +5,7 @@
  * should be used with the libcleri module.
  *
  * Source class: SiriGrammar
- * Created at: 2017-08-14 15:06:00
+ * Created at: 2018-05-16 16:15:46
  */
 
 #include "siri/grammar/grammar.h"
@@ -31,8 +31,10 @@ cleri_grammar_t * compile_grammar(void)
     cleri_t * r_comment = cleri_regex(CLERI_GID_R_COMMENT, "^#.*");
     cleri_t * k_access = cleri_keyword(CLERI_GID_K_ACCESS, "access", CLERI_CASE_SENSITIVE);
     cleri_t * k_active_handles = cleri_keyword(CLERI_GID_K_ACTIVE_HANDLES, "active_handles", CLERI_CASE_SENSITIVE);
+    cleri_t * k_active_tasks = cleri_keyword(CLERI_GID_K_ACTIVE_TASKS, "active_tasks", CLERI_CASE_SENSITIVE);
     cleri_t * k_address = cleri_keyword(CLERI_GID_K_ADDRESS, "address", CLERI_CASE_SENSITIVE);
     cleri_t * k_after = cleri_keyword(CLERI_GID_K_AFTER, "after", CLERI_CASE_SENSITIVE);
+    cleri_t * k_all = cleri_keyword(CLERI_GID_K_ALL, "all", CLERI_CASE_SENSITIVE);
     cleri_t * k_alter = cleri_keyword(CLERI_GID_K_ALTER, "alter", CLERI_CASE_SENSITIVE);
     cleri_t * k_and = cleri_keyword(CLERI_GID_K_AND, "and", CLERI_CASE_SENSITIVE);
     cleri_t * k_as = cleri_keyword(CLERI_GID_K_AS, "as", CLERI_CASE_SENSITIVE);
@@ -60,6 +62,7 @@ cleri_grammar_t * compile_grammar(void)
     cleri_t * k_false = cleri_keyword(CLERI_GID_K_FALSE, "false", CLERI_CASE_SENSITIVE);
     cleri_t * k_fifo_files = cleri_keyword(CLERI_GID_K_FIFO_FILES, "fifo_files", CLERI_CASE_SENSITIVE);
     cleri_t * k_filter = cleri_keyword(CLERI_GID_K_FILTER, "filter", CLERI_CASE_SENSITIVE);
+    cleri_t * k_first = cleri_keyword(CLERI_GID_K_FIRST, "first", CLERI_CASE_SENSITIVE);
     cleri_t * k_float = cleri_keyword(CLERI_GID_K_FLOAT, "float", CLERI_CASE_SENSITIVE);
     cleri_t * k_for = cleri_keyword(CLERI_GID_K_FOR, "for", CLERI_CASE_SENSITIVE);
     cleri_t * k_from = cleri_keyword(CLERI_GID_K_FROM, "from", CLERI_CASE_SENSITIVE);
@@ -74,6 +77,8 @@ cleri_grammar_t * compile_grammar(void)
         cleri_keyword(CLERI_NONE, "help", CLERI_CASE_SENSITIVE),
         cleri_token(CLERI_NONE, "?")
     );
+    cleri_t * k_idle_percentage = cleri_keyword(CLERI_GID_K_IDLE_PERCENTAGE, "idle_percentage", CLERI_CASE_SENSITIVE);
+    cleri_t * k_idle_time = cleri_keyword(CLERI_GID_K_IDLE_TIME, "idle_time", CLERI_CASE_SENSITIVE);
     cleri_t * k_info = cleri_keyword(CLERI_GID_K_INFO, "info", CLERI_CASE_SENSITIVE);
     cleri_t * k_ignore_threshold = cleri_keyword(CLERI_GID_K_IGNORE_THRESHOLD, "ignore_threshold", CLERI_CASE_SENSITIVE);
     cleri_t * k_insert = cleri_keyword(CLERI_GID_K_INSERT, "insert", CLERI_CASE_SENSITIVE);
@@ -86,6 +91,7 @@ cleri_grammar_t * compile_grammar(void)
         cleri_keyword(CLERI_NONE, "intersection", CLERI_CASE_SENSITIVE)
     );
     cleri_t * k_ip_support = cleri_keyword(CLERI_GID_K_IP_SUPPORT, "ip_support", CLERI_CASE_SENSITIVE);
+    cleri_t * k_last = cleri_keyword(CLERI_GID_K_LAST, "last", CLERI_CASE_SENSITIVE);
     cleri_t * k_length = cleri_keyword(CLERI_GID_K_LENGTH, "length", CLERI_CASE_SENSITIVE);
     cleri_t * k_libuv = cleri_keyword(CLERI_GID_K_LIBUV, "libuv", CLERI_CASE_SENSITIVE);
     cleri_t * k_limit = cleri_keyword(CLERI_GID_K_LIMIT, "limit", CLERI_CASE_SENSITIVE);
@@ -134,6 +140,7 @@ cleri_grammar_t * compile_grammar(void)
     cleri_t * k_start = cleri_keyword(CLERI_GID_K_START, "start", CLERI_CASE_SENSITIVE);
     cleri_t * k_startup_time = cleri_keyword(CLERI_GID_K_STARTUP_TIME, "startup_time", CLERI_CASE_SENSITIVE);
     cleri_t * k_status = cleri_keyword(CLERI_GID_K_STATUS, "status", CLERI_CASE_SENSITIVE);
+    cleri_t * k_stddev = cleri_keyword(CLERI_GID_K_STDDEV, "stddev", CLERI_CASE_SENSITIVE);
     cleri_t * k_string = cleri_keyword(CLERI_GID_K_STRING, "string", CLERI_CASE_SENSITIVE);
     cleri_t * k_suffix = cleri_keyword(CLERI_GID_K_SUFFIX, "suffix", CLERI_CASE_SENSITIVE);
     cleri_t * k_sum = cleri_keyword(CLERI_GID_K_SUM, "sum", CLERI_CASE_SENSITIVE);
@@ -287,7 +294,7 @@ cleri_grammar_t * compile_grammar(void)
     cleri_t * server_columns = cleri_list(CLERI_GID_SERVER_COLUMNS, cleri_choice(
         CLERI_NONE,
         CLERI_FIRST_MATCH,
-        25,
+        28,
         k_address,
         k_buffer_path,
         k_buffer_size,
@@ -303,7 +310,10 @@ cleri_grammar_t * compile_grammar(void)
         k_startup_time,
         k_status,
         k_active_handles,
+        k_active_tasks,
         k_fifo_files,
+        k_idle_percentage,
+        k_idle_time,
         k_log_level,
         k_max_open_files,
         k_mem_usage,
@@ -518,10 +528,13 @@ cleri_grammar_t * compile_grammar(void)
                 cleri_choice(
                     CLERI_NONE,
                     CLERI_FIRST_MATCH,
-                    12,
+                    15,
                     k_active_handles,
+                    k_active_tasks,
                     k_buffer_size,
                     k_fifo_files,
+                    k_idle_percentage,
+                    k_idle_time,
                     k_port,
                     k_pool,
                     k_startup_time,
@@ -730,6 +743,13 @@ cleri_grammar_t * compile_grammar(void)
         k_intersection,
         k_symmetric_difference
     );
+    cleri_t * series_all = cleri_choice(
+        CLERI_GID_SERIES_ALL,
+        CLERI_FIRST_MATCH,
+        2,
+        cleri_token(CLERI_NONE, "*"),
+        k_all
+    );
     cleri_t * series_name = cleri_dup(CLERI_GID_SERIES_NAME, string);
     cleri_t * group_name = cleri_dup(CLERI_GID_GROUP_NAME, r_grave_str);
     cleri_t * series_re = cleri_dup(CLERI_GID_SERIES_RE, r_regex);
@@ -744,7 +764,8 @@ cleri_grammar_t * compile_grammar(void)
     cleri_t * series_match = cleri_list(CLERI_GID_SERIES_MATCH, cleri_choice(
         CLERI_NONE,
         CLERI_FIRST_MATCH,
-        3,
+        4,
+        series_all,
         series_name,
         group_match,
         series_re
@@ -788,13 +809,14 @@ cleri_grammar_t * compile_grammar(void)
         k_suffix,
         string
     );
-    cleri_t * f_points = cleri_choice(
-        CLERI_GID_F_POINTS,
+    cleri_t * f_all = cleri_choice(
+        CLERI_GID_F_ALL,
         CLERI_FIRST_MATCH,
         2,
         cleri_token(CLERI_NONE, "*"),
-        k_points
+        k_all
     );
+    cleri_t * f_points = cleri_dup(CLERI_GID_F_POINTS, k_points);
     cleri_t * f_difference = cleri_sequence(
         CLERI_GID_F_DIFFERENCE,
         4,
@@ -816,7 +838,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_mean,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_median = cleri_sequence(
@@ -824,7 +846,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_median,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_median_low = cleri_sequence(
@@ -832,7 +854,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_median_low,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_median_high = cleri_sequence(
@@ -840,7 +862,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_median_high,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_sum = cleri_sequence(
@@ -848,7 +870,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_sum,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_min = cleri_sequence(
@@ -856,7 +878,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_min,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_max = cleri_sequence(
@@ -864,7 +886,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_max,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_count = cleri_sequence(
@@ -872,7 +894,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_count,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_variance = cleri_sequence(
@@ -880,7 +902,7 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_variance,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_pvariance = cleri_sequence(
@@ -888,7 +910,31 @@ cleri_grammar_t * compile_grammar(void)
         4,
         k_pvariance,
         cleri_token(CLERI_NONE, "("),
-        time_expr,
+        cleri_optional(CLERI_NONE, time_expr),
+        cleri_token(CLERI_NONE, ")")
+    );
+    cleri_t * f_stddev = cleri_sequence(
+        CLERI_GID_F_STDDEV,
+        4,
+        k_stddev,
+        cleri_token(CLERI_NONE, "("),
+        cleri_optional(CLERI_NONE, time_expr),
+        cleri_token(CLERI_NONE, ")")
+    );
+    cleri_t * f_first = cleri_sequence(
+        CLERI_GID_F_FIRST,
+        4,
+        k_first,
+        cleri_token(CLERI_NONE, "("),
+        cleri_optional(CLERI_NONE, time_expr),
+        cleri_token(CLERI_NONE, ")")
+    );
+    cleri_t * f_last = cleri_sequence(
+        CLERI_GID_F_LAST,
+        4,
+        k_last,
+        cleri_token(CLERI_NONE, "("),
+        cleri_optional(CLERI_NONE, time_expr),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * f_filter = cleri_sequence(
@@ -917,7 +963,7 @@ cleri_grammar_t * compile_grammar(void)
         cleri_choice(
             CLERI_NONE,
             CLERI_FIRST_MATCH,
-            10,
+            13,
             k_mean,
             k_median,
             k_median_high,
@@ -927,15 +973,18 @@ cleri_grammar_t * compile_grammar(void)
             k_max,
             k_count,
             k_variance,
-            k_pvariance
+            k_pvariance,
+            k_stddev,
+            k_first,
+            k_last
         ),
         cleri_token(CLERI_NONE, ")")
     );
     cleri_t * aggregate_functions = cleri_list(CLERI_GID_AGGREGATE_FUNCTIONS, cleri_choice(
         CLERI_NONE,
         CLERI_FIRST_MATCH,
-        15,
-        f_points,
+        19,
+        f_all,
         f_limit,
         f_mean,
         f_sum,
@@ -947,9 +996,13 @@ cleri_grammar_t * compile_grammar(void)
         f_count,
         f_variance,
         f_pvariance,
+        f_stddev,
+        f_first,
+        f_last,
         f_difference,
         f_derivative,
-        f_filter
+        f_filter,
+        f_points
     ), cleri_token(CLERI_NONE, "=>"), 1, 0, 0);
     cleri_t * select_aggregate = cleri_sequence(
         CLERI_GID_SELECT_AGGREGATE,
@@ -958,6 +1011,7 @@ cleri_grammar_t * compile_grammar(void)
         cleri_optional(CLERI_NONE, prefix_expr),
         cleri_optional(CLERI_NONE, suffix_expr)
     );
+    cleri_t * select_aggregates = cleri_list(CLERI_GID_SELECT_AGGREGATES, select_aggregate, cleri_token(CLERI_NONE, ","), 1, 0, 0);
     cleri_t * merge_as = cleri_sequence(
         CLERI_GID_MERGE_AS,
         4,
@@ -1397,7 +1451,7 @@ cleri_grammar_t * compile_grammar(void)
         CLERI_GID_SELECT_STMT,
         7,
         k_select,
-        cleri_list(CLERI_NONE, select_aggregate, cleri_token(CLERI_NONE, ","), 1, 0, 0),
+        select_aggregates,
         k_from,
         series_match,
         cleri_optional(CLERI_NONE, where_series),
@@ -1418,8 +1472,9 @@ cleri_grammar_t * compile_grammar(void)
         cleri_list(CLERI_NONE, cleri_choice(
             CLERI_NONE,
             CLERI_FIRST_MATCH,
-            31,
+            34,
             k_active_handles,
+            k_active_tasks,
             k_buffer_path,
             k_buffer_size,
             k_dbname,
@@ -1428,6 +1483,8 @@ cleri_grammar_t * compile_grammar(void)
             k_duration_log,
             k_duration_num,
             k_fifo_files,
+            k_idle_percentage,
+            k_idle_time,
             k_ip_support,
             k_libuv,
             k_list_limit,
@@ -1453,38 +1510,11 @@ cleri_grammar_t * compile_grammar(void)
         ), cleri_token(CLERI_NONE, ","), 0, 0, 0)
     );
     cleri_t * timeit_stmt = cleri_dup(CLERI_GID_TIMEIT_STMT, k_timeit);
-    cleri_t * help_show = cleri_keyword(CLERI_GID_HELP_SHOW, "show", CLERI_CASE_SENSITIVE);
-    cleri_t * help_revoke = cleri_keyword(CLERI_GID_HELP_REVOKE, "revoke", CLERI_CASE_SENSITIVE);
-    cleri_t * help_noaccess = cleri_keyword(CLERI_GID_HELP_NOACCESS, "noaccess", CLERI_CASE_SENSITIVE);
-    cleri_t * help_functions = cleri_keyword(CLERI_GID_HELP_FUNCTIONS, "functions", CLERI_CASE_SENSITIVE);
-    cleri_t * help_grant = cleri_keyword(CLERI_GID_HELP_GRANT, "grant", CLERI_CASE_SENSITIVE);
     cleri_t * help_access = cleri_keyword(CLERI_GID_HELP_ACCESS, "access", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_users = cleri_keyword(CLERI_GID_HELP_LIST_USERS, "users", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_pools = cleri_keyword(CLERI_GID_HELP_LIST_POOLS, "pools", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_series = cleri_keyword(CLERI_GID_HELP_LIST_SERIES, "series", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_servers = cleri_keyword(CLERI_GID_HELP_LIST_SERVERS, "servers", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_groups = cleri_keyword(CLERI_GID_HELP_LIST_GROUPS, "groups", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list_shards = cleri_keyword(CLERI_GID_HELP_LIST_SHARDS, "shards", CLERI_CASE_SENSITIVE);
-    cleri_t * help_list = cleri_sequence(
-        CLERI_GID_HELP_LIST,
-        2,
-        k_list,
-        cleri_optional(CLERI_NONE, cleri_choice(
-            CLERI_NONE,
-            CLERI_MOST_GREEDY,
-            6,
-            help_list_users,
-            help_list_pools,
-            help_list_series,
-            help_list_servers,
-            help_list_groups,
-            help_list_shards
-        ))
-    );
     cleri_t * help_alter_database = cleri_keyword(CLERI_GID_HELP_ALTER_DATABASE, "database", CLERI_CASE_SENSITIVE);
     cleri_t * help_alter_group = cleri_keyword(CLERI_GID_HELP_ALTER_GROUP, "group", CLERI_CASE_SENSITIVE);
-    cleri_t * help_alter_servers = cleri_keyword(CLERI_GID_HELP_ALTER_SERVERS, "servers", CLERI_CASE_SENSITIVE);
     cleri_t * help_alter_server = cleri_keyword(CLERI_GID_HELP_ALTER_SERVER, "server", CLERI_CASE_SENSITIVE);
+    cleri_t * help_alter_servers = cleri_keyword(CLERI_GID_HELP_ALTER_SERVERS, "servers", CLERI_CASE_SENSITIVE);
     cleri_t * help_alter_user = cleri_keyword(CLERI_GID_HELP_ALTER_USER, "user", CLERI_CASE_SENSITIVE);
     cleri_t * help_alter = cleri_sequence(
         CLERI_GID_HELP_ALTER,
@@ -1496,17 +1526,17 @@ cleri_grammar_t * compile_grammar(void)
             5,
             help_alter_database,
             help_alter_group,
-            help_alter_servers,
             help_alter_server,
+            help_alter_servers,
             help_alter_user
         ))
     );
+    cleri_t * help_count_groups = cleri_keyword(CLERI_GID_HELP_COUNT_GROUPS, "groups", CLERI_CASE_SENSITIVE);
+    cleri_t * help_count_pools = cleri_keyword(CLERI_GID_HELP_COUNT_POOLS, "pools", CLERI_CASE_SENSITIVE);
     cleri_t * help_count_series = cleri_keyword(CLERI_GID_HELP_COUNT_SERIES, "series", CLERI_CASE_SENSITIVE);
     cleri_t * help_count_servers = cleri_keyword(CLERI_GID_HELP_COUNT_SERVERS, "servers", CLERI_CASE_SENSITIVE);
-    cleri_t * help_count_groups = cleri_keyword(CLERI_GID_HELP_COUNT_GROUPS, "groups", CLERI_CASE_SENSITIVE);
     cleri_t * help_count_shards = cleri_keyword(CLERI_GID_HELP_COUNT_SHARDS, "shards", CLERI_CASE_SENSITIVE);
     cleri_t * help_count_users = cleri_keyword(CLERI_GID_HELP_COUNT_USERS, "users", CLERI_CASE_SENSITIVE);
-    cleri_t * help_count_pools = cleri_keyword(CLERI_GID_HELP_COUNT_POOLS, "pools", CLERI_CASE_SENSITIVE);
     cleri_t * help_count = cleri_sequence(
         CLERI_GID_HELP_COUNT,
         2,
@@ -1515,15 +1545,14 @@ cleri_grammar_t * compile_grammar(void)
             CLERI_NONE,
             CLERI_MOST_GREEDY,
             6,
+            help_count_groups,
+            help_count_pools,
             help_count_series,
             help_count_servers,
-            help_count_groups,
             help_count_shards,
-            help_count_users,
-            help_count_pools
+            help_count_users
         ))
     );
-    cleri_t * help_select = cleri_keyword(CLERI_GID_HELP_SELECT, "select", CLERI_CASE_SENSITIVE);
     cleri_t * help_create_group = cleri_keyword(CLERI_GID_HELP_CREATE_GROUP, "group", CLERI_CASE_SENSITIVE);
     cleri_t * help_create_user = cleri_keyword(CLERI_GID_HELP_CREATE_USER, "user", CLERI_CASE_SENSITIVE);
     cleri_t * help_create = cleri_sequence(
@@ -1538,12 +1567,11 @@ cleri_grammar_t * compile_grammar(void)
             help_create_user
         ))
     );
-    cleri_t * help_timezones = cleri_keyword(CLERI_GID_HELP_TIMEZONES, "timezones", CLERI_CASE_SENSITIVE);
     cleri_t * help_drop_group = cleri_keyword(CLERI_GID_HELP_DROP_GROUP, "group", CLERI_CASE_SENSITIVE);
-    cleri_t * help_drop_user = cleri_keyword(CLERI_GID_HELP_DROP_USER, "user", CLERI_CASE_SENSITIVE);
-    cleri_t * help_drop_server = cleri_keyword(CLERI_GID_HELP_DROP_SERVER, "server", CLERI_CASE_SENSITIVE);
     cleri_t * help_drop_series = cleri_keyword(CLERI_GID_HELP_DROP_SERIES, "series", CLERI_CASE_SENSITIVE);
+    cleri_t * help_drop_server = cleri_keyword(CLERI_GID_HELP_DROP_SERVER, "server", CLERI_CASE_SENSITIVE);
     cleri_t * help_drop_shards = cleri_keyword(CLERI_GID_HELP_DROP_SHARDS, "shards", CLERI_CASE_SENSITIVE);
+    cleri_t * help_drop_user = cleri_keyword(CLERI_GID_HELP_DROP_USER, "user", CLERI_CASE_SENSITIVE);
     cleri_t * help_drop = cleri_sequence(
         CLERI_GID_HELP_DROP,
         2,
@@ -1553,13 +1581,42 @@ cleri_grammar_t * compile_grammar(void)
             CLERI_MOST_GREEDY,
             5,
             help_drop_group,
-            help_drop_user,
-            help_drop_server,
             help_drop_series,
-            help_drop_shards
+            help_drop_server,
+            help_drop_shards,
+            help_drop_user
         ))
     );
+    cleri_t * help_functions = cleri_keyword(CLERI_GID_HELP_FUNCTIONS, "functions", CLERI_CASE_SENSITIVE);
+    cleri_t * help_grant = cleri_keyword(CLERI_GID_HELP_GRANT, "grant", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_groups = cleri_keyword(CLERI_GID_HELP_LIST_GROUPS, "groups", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_pools = cleri_keyword(CLERI_GID_HELP_LIST_POOLS, "pools", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_series = cleri_keyword(CLERI_GID_HELP_LIST_SERIES, "series", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_servers = cleri_keyword(CLERI_GID_HELP_LIST_SERVERS, "servers", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_shards = cleri_keyword(CLERI_GID_HELP_LIST_SHARDS, "shards", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list_users = cleri_keyword(CLERI_GID_HELP_LIST_USERS, "users", CLERI_CASE_SENSITIVE);
+    cleri_t * help_list = cleri_sequence(
+        CLERI_GID_HELP_LIST,
+        2,
+        k_list,
+        cleri_optional(CLERI_NONE, cleri_choice(
+            CLERI_NONE,
+            CLERI_MOST_GREEDY,
+            6,
+            help_list_groups,
+            help_list_pools,
+            help_list_series,
+            help_list_servers,
+            help_list_shards,
+            help_list_users
+        ))
+    );
+    cleri_t * help_noaccess = cleri_keyword(CLERI_GID_HELP_NOACCESS, "noaccess", CLERI_CASE_SENSITIVE);
+    cleri_t * help_revoke = cleri_keyword(CLERI_GID_HELP_REVOKE, "revoke", CLERI_CASE_SENSITIVE);
+    cleri_t * help_select = cleri_keyword(CLERI_GID_HELP_SELECT, "select", CLERI_CASE_SENSITIVE);
+    cleri_t * help_show = cleri_keyword(CLERI_GID_HELP_SHOW, "show", CLERI_CASE_SENSITIVE);
     cleri_t * help_timeit = cleri_keyword(CLERI_GID_HELP_TIMEIT, "timeit", CLERI_CASE_SENSITIVE);
+    cleri_t * help_timezones = cleri_keyword(CLERI_GID_HELP_TIMEZONES, "timezones", CLERI_CASE_SENSITIVE);
     cleri_t * help = cleri_sequence(
         CLERI_GID_HELP,
         2,
@@ -1568,20 +1625,20 @@ cleri_grammar_t * compile_grammar(void)
             CLERI_NONE,
             CLERI_MOST_GREEDY,
             14,
-            help_show,
-            help_revoke,
-            help_noaccess,
-            help_functions,
-            help_grant,
             help_access,
-            help_list,
             help_alter,
             help_count,
-            help_select,
             help_create,
-            help_timezones,
             help_drop,
-            help_timeit
+            help_functions,
+            help_grant,
+            help_list,
+            help_noaccess,
+            help_revoke,
+            help_select,
+            help_show,
+            help_timeit,
+            help_timezones
         ))
     );
     cleri_t * START = cleri_sequence(
