@@ -11,6 +11,8 @@ from test_series import TestSeries
 from test_server import TestServer
 from test_user import TestUser
 from test_compression import TestCompression
+from test_log import TestLog
+
 
 Server.BUILDTYPE = 'Release'
 
@@ -25,3 +27,4 @@ if __name__ == '__main__':
     run_test(TestSeries())
     run_test(TestServer())
     run_test(TestUser())
+    run_test(TestLog())
