@@ -61,6 +61,12 @@ DATA = {
     ],
     'one': [
         [1471254710, 1]
+    ],
+    'log': [
+        [1471254710, 'log line one'],
+        [1471254712, 'log line two'],
+        [1471254714, 'another line (three)'],
+        [1471254716, 'and yet one more'],
     ]
 }
 
@@ -74,7 +80,7 @@ class TestSelect(TestBase):
 
         self.assertEqual(
             await self.client0.insert(DATA),
-            {'success_msg': 'Successfully inserted 56 point(s).'})
+            {'success_msg': 'Successfully inserted 60 point(s).'})
 
         self.assertEqual(
             await self.client0.query(
@@ -214,10 +220,30 @@ class TestSelect(TestBase):
             await self.client0.query('select * from "one"'),
             {'one': [[1471254710, 1]]})
 
+        self.assertEqual(
+            await self.client0.query('select * from "log"'),
+            {'log': DATA['log']})
+
+        self.assertEqual(
+            await self.client0.query('select filter(~"log") => filter(!~"one") from "log"'),
+            {'log': [DATA['log'][1]]})
+
+        self.assertEqual(
+            await self.client0.query('select filter(~"one") prefix "1-", filter(~"two") prefix "2-" from "log"'),
+            {
+                '1-log': [[1471254710, 'log line one'], [1471254716, 'and yet one more']],
+                '2-log': [[1471254712, 'log line two']]
+            })
+
         self.assertEqual(
             await self.client0.query('select difference() from "one"'),
             {'one': []})
 
+        with self.assertRaisesRegexp(
+                QueryError,
+                'Cannot use mean\(\) on string type\.'):
+            await self.client0.query('select mean(1w) from "log"')
+
         with self.assertRaisesRegexp(
                 QueryError,
                 'Group by time must be an integer value larger than zero\.'):
@@ -263,19 +289,58 @@ class TestSelect(TestBase):
                     'Query too long.'):
             await self.client0.query('select * from "{}"'.format('a' * 65535))
 
+        with self.assertRaisesRegexp(
+                    QueryError,
+                    'Error while merging points. Make sure the destination '
+                    'series name is valid.'):
+            await self.client0.query(
+                'select * from "aggr", "huge" merge as ""')
+
         self.assertEqual(
             await self.client0.query(
                 'select min(2h) prefix "min-", max(1h) prefix "max-" '
                 'from /.*/ where type == integer and name != "filter" '
                 'and name != "one"'
                 'merge as "int_min_max" using median_low(1) => difference()'),
-            {   'max-int_min_max': [
+                {'max-int_min_max': [
                     [1447254000, 3], [1447257600, -1], [1471255200, -532]],
                 'min-int_min_max': [
                     [1447257600, -477], [1471255200, -54]]})
 
         await self.client0.query('select derivative() from "equal ts"')
 
+        self.assertEqual(
+            await self.client0.query('select first() from *'),
+            {k: [v[0]] for k, v in DATA.items()})
+
+        self.assertEqual(
+            await self.client0.query('select last() from *'),
+            {k: [v[-1]] for k, v in DATA.items()})
+
+        self.assertEqual(
+            await self.client0.query('select count() from *'),
+            {k: [[v[-1][0], len(v)]] for k, v in DATA.items()})
+
+        self.assertEqual(
+            await self.client0.query('select mean() from "aggr"'),
+            {'aggr': [[
+                DATA['aggr'][-1][0],
+                sum([x[1] for x in DATA['aggr']]) / len(DATA['aggr'])]]})
+
+        self.assertAlmostEqual(
+            await self.client0.query('select stddev() from "aggr"'),
+            {'aggr': [[
+                DATA['aggr'][-1][0],
+                147.07108914792838]]})
+
+        self.assertAlmostEqual(
+            await self.client0.query('select stddev(1h) from "aggr"'),
+            {"aggr": [
+                [1447250400, 1.8165902124584952],
+                [1447254000, 185.46409846162092],
+                [1447257600, 2.6457513110645907]]})
+
+
         # test prefix, suffex
         result = await self.client0.query(
                 'select sum(1d) prefix "sum-" suffix "-sum", '
